<?php
/**
 * @brief		Bulk mail extension
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	Tutorials
 * @since		26 Jun 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\tutorials\extensions\core\BulkMail;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Bulk mail extension
 */
class _Categories
{
	/** 
	 * Get Setting Field
	 *
	 * @param	mixed	$criteria	Value returned from the save() method
	 * @return	array 	Array of form elements
	 */
	public function getSettingField( $criteria )
	{
		return array(
			new \IPS\Helpers\Form\Node( 'bmf_tutorials_cats', isset( $criteria['bmf_tutorials_cats'] ) ? array_filter( array_map( function( $val )
			{
				try
				{
					return \IPS\tutorials\Category::load( $val );
				}
				catch ( \OutOfRangeException $e )
				{
					return NULL;
				}
			}, explode( ',', $criteria['bmf_tutorials_cats'] ) ) ) : 0, FALSE, array( 'zeroVal' => 'all_categories', 'multiple' => TRUE, 'class' => '\IPS\tutorials\Category', 'permissionCheck' => 'add' ) )
		);
	}
	
	/**
	 * Save the filter data
	 *
	 * @param	array	$post	Form values
	 * @return	mixed			False, or an array of data to use later when filtering the members
	 * @throws \LogicException
	 */
	public function save( $post )
	{
		if ( is_array( $post['bmf_tutorials_cats'] ) )
		{
			$ids = array();
			foreach ( $post['bmf_tutorials_cats'] as $cat )
			{
				$ids[] = $cat->id;
			}
			
			return array( 'bmf_tutorials_cats' => implode( ',', $ids ) );
		}

		return FALSE;
	}
	
	/**
	 * Get where clause to add to the member retrieval database query
	 *
	 * @param	mixed				$data	The array returned from the save() method
	 * @return	string|array|NULL	Where clause
	 */
	public function getQueryWhereClause( $data )
	{
		if ( isset( $data['bmf_tutorials_cats'] ) and $data['bmf_tutorials_cats'] )
		{
			return 'tutotable.a_cat IS NOT NULL';
		}

		return NULL;
	}
	
	/**
	 * Callback for member retrieval database query
	 * Can be used to set joins
	 *
	 * @param	mixed			$data	The array returned from the save() method
	 * @param	\IPS\Db\Query	$query	The query
	 * @return	void
	 */
	public function queryCallback( $data, &$query )
	{
		if ( isset( $data['bmf_tutorials_cats'] ) and $data['bmf_tutorials_cats'] )
		{
			
			$query->join( array( 'tutorials_articles', 'tutotable' ), "tutotable.a_mid=core_members.member_id AND " . \IPS\Db::i()->in( 'tutotable.a_cat', explode( ',', $data['bmf_tutorials_cats'] ) ) );
		}
	}
}