<?php
/**
 * @brief		Bulk mail extension
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	
 * @since		29 Dec 2014
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\tutorials\extensions\core\BulkMail;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Bulk mail extension
 */
class _DateAdded
{
	/** 
	 * Get Setting Field
	 *
	 * @param	mixed	$criteria	Value returned from the save() method
	 * @return	array 	Array of form elements
	 */
	public function getSettingField( $criteria )
	{
		return array(
					new \IPS\Helpers\Form\DateRange( 'bmf_tutorials_added', isset( $criteria['range'] ) ? $criteria['range'] : '', FALSE ),
		);
	}
	
	/**
	 * Save the filter data
	 *
	 * @param	array	$post	Form values
	 * @return	mixed			False, or an array of data to use later when filtering the members
	 * @throws \LogicException
	 */
	public function save( $post )
	{
		return ( empty($post['bmf_tutorials_added']) ) ? FALSE : array( 'range' => $post['bmf_tutorials_added'] );
	}
	
	/**
	 * Get where clause to add to the member retrieval database query
	 *
	 * @param	mixed				$data	The array returned from the save() method
	 * @return	string|array|NULL	Where clause
	 */
	public function getQueryWhereClause( $data )
	{
		return NULL;
	}
	
	/**
	 * Callback for member retrieval database query
	 * Can be used to set joins
	 *
	 * @param	mixed			$data	The array returned from the save() method
	 * @param	\IPS\Db\Query	$query	The query
	 * @return	void
	 */
	public function queryCallback( $data, &$query )
	{
		if( !empty($data['range']) AND !empty($data['range']['end']) )
		{
			$start	= strtotime( $data['range']['start']['date'] );
			$end	= strtotime( $data['range']['end']['date'] );

			$where[] = 'tutorials_articles.a_date BETWEEN '.$start.' AND '.$end;
			$where[] = 'tutorials_articles.a_mid=core_members.member_id';

			$filter = implode( " AND ", $where );

			$query->join( 'tutorials_articles', $filter );			
		}
	}
}