<?php
/**
 * @brief		File Storage Extension: Icon
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	Tutorials
 * @since		01 Jul 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\tutorials\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * File Storage Extension: Icon
 */
class _Icon
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'tutorials_cats', 'c_icon IS NOT NULL' )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\UnderflowException					When file record doesn't exist. Indicating there are no more files to move
	 * @return	void|int							An offset integer to use on the next cycle, or nothing
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$cat = \IPS\tutorials\Category::constructFromData( \IPS\Db::i()->select( '*', 'tutorials_cats', 'c_icon IS NOT NULL', 'c_id', array( $offset, 1 ) )->first() );
		
		try
		{
			$cat->c_icon = \IPS\File::get( $oldConfiguration ?: 'tutorials_Icon', $cat->c_icon )->move( $storageConfiguration );
			$cat->save();
		}
		catch( \Exception $e )
		{
			/* Any issues are logged */
		}
	}

	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		$cat = \IPS\tutorials\Category::constructFromData( \IPS\Db::i()->select( '*', 'tutorials_cats', 'c_icon IS NOT NULL', 'c_id', array( $offset, 1 ) )->first() );
		
		if ( $new = \IPS\File::repairUrl( $cat->c_icon ) )
		{
			$cat->c_icon = $new;
			$cat->save();
		}
	}

	/**
	 * Check if a file is valid
	 *
	 * @param	\IPS\Http\Url	$file		The file to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			\IPS\Db::i()->select( 'c_id', 'v', array( 'c_icon=?', $file ) )->first();
			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'tutorials_cats', "c_icon IS NOT NULL" ) as $cat )
		{
			try
			{
				\IPS\File::get( 'tutorials_Icon', $cat['c_icon'] )->delete();
			}
			catch( \Exception $e ){}
		}
	}
}