<?php
/**
 * @brief		File Storage Extension: Image
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Tutorials
 * @since		26 Oct 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\tutorials\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * File Storage Extension: Image
 */
class _Image
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'tutorials_articles' )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\UnderflowException					When file record doesn't exist. Indicating there are no more files to move
	 * @return	void|int							An offset integer to use on the next cycle, or nothing
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$article = \IPS\tutorials\Article::constructFromData( \IPS\Db::i()->select( '*', 'tutorials_articles', 'a_image IS NOT NULL', 'a_id', array( $offset, 1 ) )->first() );
		
		try
		{
			$article->image = \IPS\File::get( $oldConfiguration ?: 'tutorials_Image', $article->image )->move( $storageConfiguration );
			$article->save();
		}
		catch( \Exception $e )
		{
			/* Any issues are logged */
		}
	}

	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		$article = \IPS\tutorials\Article::constructFromData( \IPS\Db::i()->select( '*', 'tutorials_articles', 'a_image IS NOT NULL', 'a_id', array( $offset, 1 ) )->first() );
		
		if ( $new = \IPS\File::repairUrl( $article->image ) )
		{
			$article->image = $new;
			$article->save();
		}
	}

	/**
	 * Check if a file is valid
	 *
	 * @param	\IPS\Http\Url	$file		The file to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			\IPS\Db::i()->select( '*', 'tutorials_articles', array( 'a_image=?', (string) $file ) )->first();
			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'tutorials_articles', "a_image IS NOT NULL" ) as $tutorial )
		{
			try
			{
				\IPS\File::get( 'tutorials_Image', $tutorial['a_image'] )->delete();
				\IPS\File::get( 'tutorials_Image', $tutorial['a_thumb'] )->delete();
			}
			catch( \Exception $e ){}
		}
	}
}