<?php
/**
 * @brief		File Storage Extension: catImage
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Tutorials
 * @since		28 Oct 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\tutorials\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * File Storage Extension: catImage
 */
class _catImage
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'tutorials_cats' )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\UnderflowException					When file record doesn't exist. Indicating there are no more files to move
	 * @return	void|int							An offset integer to use on the next cycle, or nothing
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$record = \IPS\Db::i()->select( '*', 'tutorials_cats', array(), 'c_id', array( $offset, 1 ) )->first();

		try
		{
			$file = \IPS\File::get( $oldConfiguration ?: 'tutorials_catImage', $record['c_image'] )->move( $storageConfiguration );
			
			if ( (string) $file != $record['c_image'] )
			{
				\IPS\Db::i()->update( 'tutorials_cats', array( 'c_image' => (string) $file ), array( 'c_id=?', $record['c_id'] ) );
			}
		}
		catch( \Exception $e )
		{
			/* Any issues are logged */
		}
	}

	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		$record = \IPS\Db::i()->select( '*', 'tutorials_cats', array(), 'c_id', array( $offset, 1 ) )->first();
		
		if ( $new = \IPS\File::repairUrl( $record['c_image'] ) )
		{
			\IPS\Db::i()->update( 'tutorials_cats', array( 'c_image' => $new ), array( 'c_id=?', $record['c_id'] ) );
		}
	}

	/**
	 * Check if a file is valid
	 *
	 * @param	\IPS\Http\Url	$file		The file to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			\IPS\Db::i()->select( '*', 'tutorials_cats', array( 'c_image=?', (string) $file ) )->first();
			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'tutorials_cats', "c_image IS NOT NULL" ) as $tutorial )
		{
			try
			{
				\IPS\File::get( 'tutorials_catImage', $tutorial['c_image'] )->delete();
			}
			catch( \Exception $e ){}
		}
	}
}