<?php


namespace IPS\tutorials\modules\admin\stats;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _submitters extends \IPS\Dispatcher\Controller
{
	const PER_PAGE = 25;

	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'tut_submitters_manage' );
		parent::execute();
	}

	protected function manage()
	{
		$values = NULL;
		$where = array();
		
		if ( isset( \IPS\Request::i()->form ) )
		{
			$form = new \IPS\Helpers\Form( 'form', 'go' );
			$form->add( new \IPS\Helpers\Form\DateRange( 'stats_date_range' ) );
			
			if ( $values = $form->values() )
			{
				if ( $values['stats_date_range']['start'] )
				{
					$where[] = array( 'a_date>?', $values['stats_date_range']['start']->getTimestamp() );
				}
				if ( $values['stats_date_range']['end'] )
				{
					$where[] = array( 'a_date<?', $values['stats_date_range']['end']->getTimestamp() );
				}
			}
			else
			{
				\IPS\Output::i()->output = $form;
				return;
			}
		}
		
		$page = isset( \IPS\Request::i()->page ) ? \intval( \IPS\Request::i()->page ) : 1;
		if( $page < 1 )
		{
			$page = 1;
		}

		try
		{
			$total = \IPS\Db::i()->select( 'COUNT( DISTINCT( a_mid ) )', 'tutorials_articles', $where )->first();
		}
		catch ( \UnderflowException $e )
		{
			$total = 0;
		}

		//$select = \IPS\Db::i()->select( 'a_mid, COUNT(*) as articles, core_members.*', 'tutorials_articles', $where, 'articles DESC', array( ( $page - 1 ) * static::PER_PAGE, static::PER_PAGE ), 'a_mid', NULL, \IPS\Db::SELECT_SQL_CALC_FOUND_ROWS )->join( 'core_members', 'core_members.member_id=tutorials_articles.a_mid' );

		if( $total > 0 )
		{
			$select	= \IPS\Db::i()->select( 'a_mid, count(*) as articles', 'tutorials_articles', $where, 'articles DESC', array( ( $page - 1 ) * static::PER_PAGE, static::PER_PAGE ), 'a_mid' )->join( 'core_members', 'core_members.member_id=tutorials_articles.a_mid' );
			$mids	= array();

			foreach( $select as $row )
			{
				$mids[] = $row['a_mid'];
			}

			$members = array();
			if( \count( $mids ) )
			{
				$members = iterator_to_array( \IPS\Db::i()->select( '*', 'core_members', array( \IPS\Db::i()->in( 'member_id', $mids ) ) )->setKeyField( 'member_id' ) );
			}

			$pagination = \IPS\Theme::i()->getTemplate( 'global', 'core', 'global' )->pagination(
				\IPS\Http\Url::internal( 'app=tutorials&module=stats&controller=submitters' )->setQueryString( $values ),
				ceil( $select->count( TRUE ) / static::PER_PAGE ),
				$page,
				static::PER_PAGE,
				FALSE
			);
			
			\IPS\Output::i()->sidebar['actions'] = array(
				'settings'	=> array(
					'title'		=> 'stats_date_range',
					'icon'		=> 'calendar',
					'link'		=> \IPS\Http\Url::internal( 'app=tutorials&module=stats&controller=submitters&form=1' )->setQueryString( $values ),
					'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack('stats_date_range') )
				)
			);
		
			\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('stats')->submittersTable( $select, $pagination, $members );
			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('menu__tutorials_stats_submitters');
		}
		else
		{
			\IPS\Output::i()->output .= \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( \IPS\Member::loggedIn()->language()->addToStack('menu__tutorials_stats_submitters'), \IPS\Member::loggedIn()->language()->addToStack('no_results'), FALSE , 'ipsPad', NULL, TRUE );
		}
	}
}