<?php

namespace IPS\tutorials\modules\admin\tutorials;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * settings
 */
class _settings extends \IPS\Dispatcher\Controller
{
	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'settings_manage' );

		$form = new \IPS\Helpers\Form;

		$form->addTab( 'editor_settings_general' );
		$form->addHeader('global_options');
		$form->add( new \IPS\Helpers\Form\YesNo( 'tutorials_formatnames', \IPS\Settings::i()->tutorials_formatnames ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'tutorials_rss', \IPS\Settings::i()->tutorials_rss, FALSE, array( 'togglesOn' => array( 'tutorials_rss_nr' ) ), NULL, NULL, NULL, 'tutorials_rss' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'tutorials_rss_nr', \IPS\Settings::i()->tutorials_rss_nr, FALSE, array( 'min' => 1, 'max' => 25 ), NULL, NULL, '', 'tutorials_rss_nr' ) );

		$form->addTab( 'tutorials_index_view' );
		$form->addHeader('tutorials_index');
		$indexType = array(
			'portal' 		=> \IPS\Member::loggedIn()->language()->addToStack('tutorials_index_landing'),
			'categories'	=> \IPS\Member::loggedIn()->language()->addToStack('tutorials_index_categories')
		);
		$form->add( new \IPS\Helpers\Form\Radio( 'tutorials_index_type', \IPS\Settings::i()->tutorials_index_type, FALSE, array( 'options' => $indexType, 'parse' => 'normal' ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'tutorials_showsidebar', \IPS\Settings::i()->tutorials_showsidebar ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'tutorials_showmenucat', \IPS\Settings::i()->tutorials_showmenucat ) );
		$form->add( new \IPS\Helpers\Form\Number( 'tutorials_index_cat_lines', \IPS\Settings::i()->tutorials_index_cat_lines, FALSE, array( 'min' => 1 ) ) );

		$form->addTab( 'tutorials_category_view' );
		$form->addHeader('category_options');
		$form->add( new \IPS\Helpers\Form\YesNo( 'tutorials_filterbyletters', \IPS\Settings::i()->tutorials_filterbyletters ) );
		$form->add( new \IPS\Helpers\Form\Number( 'tutorials_artspp', \IPS\Settings::i()->tutorials_artspp ? \IPS\Settings::i()->tutorials_artspp : 25, FALSE, array( 'min' => 5, 'max' => 50 ), NULL, NULL, '', 'tutorials_artspp' ) );

		$form->addTab( 'tutorials_tutorial_view' );
		$form->addHeader('article_options');
		$form->add( new \IPS\Helpers\Form\YesNo( 'tutorials_logvisit_author', \IPS\Settings::i()->tutorials_logvisit_author ) );
		$form->add( new \IPS\Helpers\Form\Number( 'tutorials_com_pp', \IPS\Settings::i()->tutorials_com_pp ? \IPS\Settings::i()->tutorials_com_pp : 5, FALSE, array( 'min' => 3, 'max' => 10 ), NULL, NULL, '', 'tutorials_com_pp' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'tutorials_rev_pp', \IPS\Settings::i()->tutorials_rev_pp ? \IPS\Settings::i()->tutorials_rev_pp : 5, FALSE, array( 'min' => 3, 'max' => 10 ), NULL, NULL, '', 'tutorials_rev_pp' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'tutorials_maxpreview', \IPS\Settings::i()->tutorials_maxpreview ? \IPS\Settings::i()->tutorials_maxpreview : 350, FALSE, array( 'min' => 10, 'max' => 500 ), NULL, NULL, '', 'tutorials_maxpreview' ) );

		/* IP.Board integration */
		if ( \IPS\Application::appIsEnabled( 'forums' ) )
		{
			$form->addTab( 'tutorials_support_topic' );
			$form->addHeader('tutorials_topics_options');
			$form->add( new \IPS\Helpers\Form\Node( 'tutorials_topics_forum', \IPS\Settings::i()->tutorials_topics_forum ? \IPS\Settings::i()->tutorials_topics_forum : 0, FALSE, array( 'class' => 'IPS\forums\Forum', 'zeroVal' => 'use_cat_forum' ), NULL, NULL, NULL, 'tutorials_topics_forum' ) );
			//$form->add( new \IPS\Helpers\Form\Translatable( 'tutorials_topic_text', \IPS\Member::loggedIn()->language()->addToStack('tutorials_support_topic_text'), FALSE, array( 'app' => 'tutorials', 'key' => 'tutorials_topic_text', 'editor' => array( 'app' => 'tutorials', 'key' => 'Admin', 'autoSaveKey' => 'TopicText', 'tags' => array( '{link}' => \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_topic_text_tags_link' ), '{date_added}' => \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_topic_text_tags_added'), '{date_updated}' => \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_topic_text_tags_updated') ), 'attachIds' => array( NULL, NULL, 'tutorials_topic_text' ), 'minimize' => 'tutorials_topic_text_placeholder' ) ), NULL, NULL, NULL, 'tutorials_topic_text' ) );*/
		}

		if( \IPS\tutorials\Store::canUsePoints() )
		{
			$form->addTab( 'tutorials_tab_membersshop' );
			$form->addHeader( 'tutorials_tab_membersshop_enable' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'tutorials_mshop', \IPS\Settings::i()->tutorials_mshop, FALSE, array( 'togglesOn' => array( "{$form->id}_header_tutorials_tab_membersshop_earn", 'tutorials_mshop_points', "{$form->id}_header_tutorials_tab_membersshop_pay", 'tutorials_mshop_paypoints' ) ), NULL, NULL, NULL, 'tutorials_mshop' ) );
		}

		if ( $values = $form->values( TRUE ) )
		{
			$form->saveAsSettings( $values );
			\IPS\Session::i()->log( 'acplogs__tutorials_settings' );
		}
		
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('settings');
		\IPS\Output::i()->output = $form;
	}

	public static function canUsePoints()
	{
		if( \IPS\Application::appIsEnabled( 'membersshop' ) AND \IPS\Application::load( 'membersshop' )->long_version >= 11003 )
		{
			return TRUE;
		}

		return FALSE;
	}
}