<?php


namespace IPS\tutorials\modules\admin\tutorials;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * tools
 */
class _tools extends \IPS\Dispatcher\Controller
{
	/**
	 * @brief	Has been CSRF-protected
	 */
	public static $csrfProtected = TRUE;

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('menu__tutorials_tutorials_tools');

		$this->createTopics();
		$this->rebuildTopics();

		if ( \IPS\Application::appIsEnabled( 'forums' ) )
		{
			$this->importTopics();
		}

		$this->rebuildOldContent();
	}

	protected function createTopics()
	{
		/* Create Support Topics */
 		$form = new \IPS\Helpers\Form( 'create', 'tools_button' );
		$form->addHeader('tools_create_support_topic');
		$form->addMessage('tools_create_support_topic_msg');
		$form->hiddenValues['something'] = 'something';

		if ( $values = $form->values() )
		{
			$cats = array();

			foreach( \IPS\Db::i()->select( '*', 'tutorials_cats', 'c_forum_tut_on = 1 AND c_forum > 0' ) as $category )
			{
				$cats[] = $category['c_id'];
			}

			$_SESSION['tutorials_create_topics_cats'] = $cats;
			
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=tools&do=doCreateTopics' ) );
		}

		\IPS\Output::i()->output .= $form;
	}

	public function doCreateTopics()
	{
		$cycle 	= 25;

		\IPS\Output::i()->title  = \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_create_topics' );

		\IPS\Output::i()->output = new \IPS\Helpers\MultipleRedirect( \IPS\Http\Url::internal('app=tutorials&module=tutorials&controller=tools&do=doCreateTopics'),
		function( $data ) use ( $cycle )
		{
			$where[] = \IPS\Db::i()->in( 'a_cat', $_SESSION['tutorials_create_topics_cats'] );
			$where[] = 'a_topic = 0';
	
			$select = \IPS\Db::i()->select( '*', 'tutorials_articles', array( implode( ' AND ', $where ) ), 'a_id ASC', array( is_array( $data ) ? $data['done'] : 0, $cycle ), NULL, NULL, \IPS\Db::SELECT_SQL_CALC_FOUND_ROWS );

			$total	= $select->count( TRUE );

			if ( !$select->count() )
			{
				return NULL;
			}

			if( !is_array( $data ) )
			{
				$data = array( 'total' => $total, 'done' => 0 );
			}

			foreach( $select as $row )
			{
				try
				{
					try
					{
						$tutorial = \IPS\tutorials\Article::load( $row['a_id'] );
						//$category =	\IPS\tutorials\Category::load( $tutorial->cat );
					}
					catch( \Exception $e ){}

					/* Swap tags */
					$def  = trim( \IPS\Member::loggedIn()->language()->get('tutorials_support_topic_text') );
					$text = str_replace( array( "{link}", "{date_added}", "{date_updated}" ), array( "<a href='{$tutorial->url()}'>{$tutorial->name}</a>", \IPS\DateTime::ts( $tutorial->date ), \IPS\DateTime::ts( $tutorial->updated ) ), $def );

					$author = \IPS\Member::load( $tutorial->mid );
					$topic  = \IPS\forums\Topic::createItem( $author, $tutorial->ipaddress, \IPS\DateTime::ts( time() ), \IPS\forums\Forum::load( \IPS\tutorials\Category::load( $tutorial->cat )->forum ), FALSE );
					$topic->title = $tutorial->name;
					$topic->topic_archive_status = \IPS\forums\Topic::ARCHIVE_EXCLUDE;
					$topic->save();

					$post = \IPS\forums\Topic\Post::create( $topic, $text, TRUE, NULL,  NULL, $author );
			   		$topic->topic_firstpost = $post->pid;
			   		$topic->save();

			   		$tutorial->topic = $topic->tid;
			   		$tutorial->save();
				}
				catch( \Exception $e ) {}
			}

			$data['done'] += $cycle;

			return array( $data, \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_creating_topics' ),( $data['done'] / $data['total'] ) * 100 );
		}, function()
		{
			/* Finished */
			$_SESSION['tutorials_create_topics_cats']	= NULL;
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal('app=tutorials&module=tutorials&controller=tools&do=manage'), 'completed' );
		} );
	}

	protected function rebuildTopics()
	{
		/* Rebuld Support Topics */
 		$form1 = new \IPS\Helpers\Form( 'rebuild', 'tools_button' );
		$form1->addHtml('<br>');
		$form1->addHeader('tools_rebuild_support_topic');
		$form1->addMessage('tools_rebuild_support_topic_msg');
		$form1->hiddenValues['something'] = 'something';

		if ( $values = $form1->values() )
		{
			$where = 'a_approved = 1 AND a_topic > 0';
			$_SESSION['tutorials_rebuild_topics'] = $where;
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=tools&do=rebuildTopicsCycle' ) );
		}

		\IPS\Output::i()->output .= $form1;
	}

	public function rebuildTopicsCycle()
	{
		$cycle 	= 50;

		\IPS\Output::i()->title  = \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_rebuild_topics' );

		\IPS\Output::i()->output = new \IPS\Helpers\MultipleRedirect( \IPS\Http\Url::internal('app=tutorials&module=tutorials&controller=tools&do=rebuildTopicsCycle'),
		function( $data ) use ( $cycle )
		{
			$select = \IPS\Db::i()->select( '*', 'tutorials_articles', $_SESSION['tutorials_rebuild_topics'], 'a_id ASC', array( is_array( $data ) ? $data['done'] : 0, $cycle ), NULL, NULL, \IPS\Db::SELECT_SQL_CALC_FOUND_ROWS );

			$total	= $select->count( TRUE );

			if ( !$select->count() )
			{
				return NULL;
			}

			if( !is_array( $data ) )
			{
				$data = array( 'total' => $total, 'done' => 0 );
			}

			foreach( $select as $row )
			{
				try
				{
					try
					{
						$tutorial = \IPS\tutorials\Article::load( $row['a_id'] );
						$topic 	  = \IPS\forums\Topic::load( $row['a_topic'] );
					}
					catch( \Exception $e ) {}

					$firstPost = $topic->comments( 1 );
					$content = \IPS\Theme::i()->getTemplate( 'submit', 'tutorials', 'front' )->topic( $tutorial );
					\IPS\Member::loggedIn()->language()->parseOutputForDisplay( $content );
					$firstPost->post = $content;
					$firstPost->save();
				}
				catch( \Exception $e ) {}
			}

			$data['done'] += $cycle;

			return array( $data, \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_rebuilding_topics' ),( $data['done'] / $data['total'] ) * 100 );
		}, function()
		{
			/* Finished */
			$_SESSION['tutorials_rebuild_topics']	= NULL;
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal('app=tutorials&module=tutorials&controller=tools&do=manage'), 'completed' );
		} );
	}

	protected function importTopics()
	{
		/* Import Topics as new Tutorials */
 		$form2 = new \IPS\Helpers\Form( 'import', 'tools_button' );
		$form2->addHtml('<br>');
		$form2->addHeader('tools_import_topics');
		$form2->addMessage('tools_import_topics_msg');
		$form2->add( new \IPS\Helpers\Form\Node(  'a_container',  NULL, TRUE, array(
			'class'		      => '\IPS\tutorials\Category',
			'disabled'	      => false,
			'permissionCheck' => 'add'
		) ) );
		
 		$form2->add( new \IPS\Helpers\Form\Stack( 'tools_import_topics_tid', NULL, TRUE ) );

		if ( $values = $form2->values() )
		{
			$cat = $values['a_container'];
			foreach( $values['tools_import_topics_tid'] as $tid )
			{
				try
				{
					$topic = \IPS\forums\Topic::loadAndCheckPerms( $tid );
				}
				catch ( \OutOfRangeException $e )
				{
					\IPS\Output::i()->error( 'tools_import_topics_error_tid', 'TUTORIALS-IMPORT/1', 404, '' );
				}

				$tutorial = \IPS\tutorials\Article::createItem( \IPS\Member::loggedIn(), \IPS\Member::loggedIn()->ip_address, \IPS\DateTime::ts( time() ), $cat, FALSE );
				$tutorial->name 	= $topic->title;
				$tutorial->content 	= \IPS\forums\Topic\Post::load( $topic->topic_firstpost )->post;
				$tutorial->save();

				$forum = \IPS\Settings::i()->tutorials_topics_forum > 0 ? \IPS\Settings::i()->tutorials_topics_forum : $cat->forum;

				if ( $forum )
				{
					$tutorial->topic = $tid;
					$tutorial->save();

					$topic->title = $cat->prefix . ' ' . $tutorial->name . ' ' . $cat->suffix;
					$topic->save();

					$firstPost = $topic->comments( 1 );
					$content = \IPS\Theme::i()->getTemplate( 'submit', 'tutorials', 'front' )->topic( $tutorial );
					\IPS\Member::loggedIn()->language()->parseOutputForDisplay( $content );
					$firstPost->post = $content;
					$firstPost->save();
				}
			}

			$cat->setLastArticle();
			$cat->save();

			\IPS\Output::i()->redirect( \IPS\Http\Url::internal('app=tutorials&module=tutorials&controller=tools'), 'completed' );
		}

		\IPS\Output::i()->output .= $form2;
	}

	protected function rebuildOldContent()
	{
		/* Rebuild Old Content */
 		$form3 = new \IPS\Helpers\Form( 'rebuild_content', 'tools_button' );
		$form3->addHtml('<br>');
		$form3->addHeader('tools_rebuild_old_content');
		$form3->addMessage('tools_rebuild_old_content_msg');
		$form3->hiddenValues['something'] = 'something';

		if ( $values = $form3->values() )
		{
		   \IPS\Task::queue( 'core', 'RebuildPosts', array( 'class' => 'IPS\tutorials\Article' ), 2 );
		   \IPS\Task::queue( 'core', 'RebuildPosts', array( 'class' => 'IPS\tutorials\Article\Comment' ), 2 );
		   \IPS\Task::queue( 'core', 'RebuildNonContentPosts', array( 'extension' => 'tutorials_Categories' ), 2 );
			
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=core&module=overview&controller=dashboard' ) );
		}

		\IPS\Output::i()->output .= $form3;
	}
}