<?php

namespace IPS\tutorials\modules\front\tutorials;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _browse extends \IPS\Dispatcher\Controller
{
	protected function manage()
	{
		if ( \IPS\Request::i()->id == 'clubs' and \IPS\Settings::i()->club_nodes_in_apps )
		{
			\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=browse&do=categories', 'front', 'tutorials_categories' ), array(), 'loc_tutorials_browsing_categories' );
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=browse&do=categories', 'front', 'tutorials_categories' ), \IPS\Member::loggedIn()->language()->addToStack('tutorials_categories') );
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=browse&id=clubs', 'front', 'tutorials_clubs' ), \IPS\Member::loggedIn()->language()->addToStack('club_node_tutorials') );
			\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('club_node_tutorials');
			\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'browse' )->categories( TRUE );
		}
		else
		{
			try
			{
				$this->_category( \IPS\tutorials\Category::loadAndCheckPerms( \IPS\Request::i()->id, 'read' ) );
			}
			catch ( \OutOfRangeException $e )
			{
				\IPS\Output::i()->error( 'node_error', 'TUTORIALS - CATEGORIES/1', 404, '' );
			}
		}
	}

	protected function categories()
	{
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=index&do=categories', 'front', 'tutorials_categories' ), array(), 'loc_tutorials_browsing_categories' );
		
		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('tutorials');
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'browse' )->categories();
	}

	protected function _category( $category )
	{
		$_count = \IPS\tutorials\Article::getItemsWithPermission( array( array( \IPS\tutorials\Article::$databasePrefix . \IPS\tutorials\Article::$databaseColumnMap['container'] . '=?', $category->_id ) ), NULL, 1, 'read', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, FALSE, FALSE, FALSE, TRUE );

		if( !$_count )
		{
			if ( $club = $category->club() )
			{
				\IPS\core\FrontNavigation::$clubTabActive = TRUE;
				\IPS\Output::i()->breadcrumb = array();
				\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=clubs&controller=directory', 'front', 'clubs_list' ), \IPS\Member::loggedIn()->language()->addToStack('module__core_clubs') );
				\IPS\Output::i()->breadcrumb[] = array( $club->url(), $club->name );
				\IPS\Output::i()->breadcrumb[] = array( $category->url(), $category->_title );
				
				if ( \IPS\Settings::i()->clubs_header == 'sidebar' )
				{
					\IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'clubs', 'core' )->header( $club, $category, 'sidebar' );
				}
			}
			else
			{
				foreach ( $category->parents() as $parent )
				{
					\IPS\Output::i()->breadcrumb[] = array( $parent->url(), $parent->_title );
				}
				\IPS\Output::i()->breadcrumb[] = array( NULL, $category->_title );
			}

			$table = \IPS\Theme::i()->getTemplate( 'browse' )->noArticles( $category );
		}
		else
		{
			$table = new \IPS\Helpers\Table\Content( 'IPS\tutorials\Article', $category->url(), NULL, $category );
			$table->classes = array( 'ipsDataList', 'ipsDataList_zebra' );
			$table->limit = \intval( \IPS\Settings::i()->tutorials_artspp ) ? \IPS\Settings::i()->tutorials_artspp : 25;

			$member = \IPS\Member::loggedIn()->member_id;

			if ( $member )
			{
				if( \IPS\tutorials\Article::modPermission( 'unhide', NULL, $category ) )
				{
					$table->filters = array(
						'tutorials_pending'		=> "( tutorials_articles.a_approved = 0 )",
						'tutorials_hidden'		=> "( tutorials_articles.a_approved = -1 )",
						'tutorials_featured'	=> "( tutorials_articles.a_featured = 1 )",
						'tutorials_locked'		=> "( tutorials_articles.a_locked = 1 )",
						'tutorials_unlocked'	=> "( tutorials_articles.a_locked = 0 )",
						'tutorials_mine'		=> "( tutorials_articles.a_mid = {$member} )",
						'tutorials_others'		=> "( tutorials_articles.a_mid <> {$member} )",
					);
				}
				else
				{
					$table->filters = array(
						'tutorials_featured'	=> "( tutorials_articles.a_featured = 1 )",
						'tutorials_locked'		=> "( tutorials_articles.a_locked = 1 )",
						'tutorials_unlocked'	=> "( tutorials_articles.a_locked = 0 )",
						'tutorials_mine'		=> "( tutorials_articles.a_mid = {$member} )",
						'tutorials_others'		=> "( tutorials_articles.a_mid <> {$member} )",
					);
				}
			}
			else
			{
				$table->filters = array(
					'tutorials_featured'	=> "( tutorials_articles.a_featured = 1 )",
					'tutorials_locked'		=> "( tutorials_articles.a_locked = 1 )",
					'tutorials_unlocked'	=> "( tutorials_articles.a_locked = 0 )",
				);
			}
		}

		$permissions = $category->permissions();
		\IPS\Session::i()->setLocation( $category->url(), explode( ",", $permissions['perm_view'] ), 'loc_tutorials_viewing_category', array( "tutorials_category_{$category->id}" => TRUE ) );
				
		\IPS\Output::i()->title		= $category->_title;

		\IPS\Output::i()->contextualSearchOptions[ \IPS\Member::loggedIn()->language()->addToStack( 'search_contextual_item', FALSE, array( 'sprintf' => array( \IPS\Member::loggedIn()->language()->addToStack( 'category' ) ) ) ) ] = array( 'type' => 'tutorials_article', 'nodes' => $category->_id );

		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'browse' )->category( $category, (string) $table );
	}

	protected function markRead()
	{
		\IPS\Session::i()->csrfCheck();

		try
		{
			$category = \IPS\tutorials\Category::load( \IPS\Request::i()->id );

			\IPS\tutorials\Article::markContainerRead( $category, NULL, FALSE );

			\IPS\Output::i()->redirect( $category->url() );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-MARK READ/1', 403, 'no_module_permission_guest' );
		}
	}

	protected function markReadWithsSubcats()
	{
		\IPS\Session::i()->csrfCheck();

		try
		{
			$category = \IPS\tutorials\Category::load( \IPS\Request::i()->id );

			\IPS\tutorials\Article::markContainerRead( $category, NULL, FALSE );
			if( $category->hasChildren( NULL ) )
			{
				foreach( $category->children() as $subCategory )
				{
					\IPS\tutorials\Article::markContainerRead( $subCategory, NULL, FALSE );
				}
			}

			\IPS\Output::i()->redirect( $category->url() );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-MARK READ/2', 403, 'no_module_permission_guest' );
		}
	}
}