<?php

namespace IPS\tutorials\modules\front\tutorials;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Tutorials Index
 */
class _index extends \IPS\Dispatcher\Controller
{
	/**
	 * Route
	 *
	 * @return	void
	 */
	protected function manage()
	{
		/* Add RSS feed */
		if ( \IPS\Settings::i()->tutorials_rss )
		{
			\IPS\Output::i()->rssFeeds['tutorials_rss_title'] = \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=index&do=rss', 'front', 'tutorials_rss' );
		}
	
		/* Online User Location */
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=tutorials', 'front', 'tutorials' ), array(), 'loc_tutorials_browsing' );

		/* Display Categories Sidebar */
		if( \IPS\Settings::i()->tutorials_showsidebar )
		{
			\IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'browse' )->indexSidebar( \IPS\tutorials\Category::canOnAny('add') );
		}

		\IPS\Output::i()->title		= \IPS\Member::loggedIn()->language()->addToStack('__app_tutorials');
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'browse' )->index();
	}
	
	/**
	 * Latest Tutorials RSS
	 *
	 * @return	void
	 */
	protected function rss()
	{
		if( !\IPS\Settings::i()->tutorials_rss )
		{
			\IPS\Output::i()->error( 'rss_offline', 'TUTORIALS-OFFLINE/1', 403, 'rss_offline_tutorials_admin' );
		}

		$document = \IPS\Xml\Rss::newDocument( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=index', 'front', 'tutorials' ), \IPS\Member::loggedIn()->language()->addToStack('tutorials_rss_title'), \IPS\Member::loggedIn()->language()->addToStack('tutorials_rss_title') );

		foreach( \IPS\tutorials\Article::getItemsWithPermission( array( 'tutorials_articles.a_approved=1' ), 'a_date DESC', \IPS\Settings::i()->tutorials_rss_nr, 'read' ) as $article )
		{
			if( $article->container()->open )
			{
				$document->addItem( $article->name, $article->url(), $article->content, \IPS\DateTime::ts( $article->updated ), $article->id );
			}
		}

		/* @note application/rss+xml is not a registered IANA mime-type so we need to stick with text/xml for RSS */
		\IPS\Output::i()->sendOutput( $document->asXML(), 200, 'text/xml' );
	}

	/**
	 * Top Tutorials Contributors
	 *
	 * @retun	void
	 */
	public function topTutorialsContributors()
	{
		/* How many? */
		$limit = \intval( ( isset( \IPS\Request::i()->limit ) and \IPS\Request::i()->limit < 20 ) ? \IPS\Request::i()->limit : 5 );
		
		/* What timeframe? */
		$where = array( 'a_mid > 0' );
		$timeframe = 'all';
		if ( isset( \IPS\Request::i()->time ) and \IPS\Request::i()->time != 'all' )
		{
			switch ( \IPS\Request::i()->time )
			{
				case 'week':
					$where[] = array( 'a_date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1W' ) )->getTimestamp() );
					$timeframe = 'week';
					break;
				case 'month':
					$where[] = array( 'a_date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1M' ) )->getTimestamp() );
					$timeframe = 'month';
					break;
				case 'year':
					$where[] = array( 'a_date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1Y' ) )->getTimestamp() );
					$timeframe = 'year';
					break;
			}

			$topContributors = iterator_to_array( \IPS\Db::i()->select( 'a_mid as member, count(a_id) as rep', 'tutorials_articles', $where, 'rep DESC', $limit, 'member' )->setKeyField('member')->setValueField('rep') );
        }
        else
        {
			$topContributors = iterator_to_array( \IPS\Db::i()->select( 'a_mid as member, count(a_id) as rep', 'tutorials_articles', $where, 'rep DESC', $limit, 'member' )->setKeyField('member')->setValueField('rep') );
        }

		arsort( $topContributors );

		/* Load their data */	
		foreach ( \IPS\Db::i()->select( '*', 'core_members', \IPS\Db::i()->in( 'member_id', array_keys( $topContributors ) ) ) as $member )
		{
			\IPS\Member::constructFromData( $member );
		}

		/* Render */
		$output = \IPS\Theme::i()->getTemplate( 'widgets' )->topContributorsTutorialsRow( $topContributors, $timeframe, \IPS\Request::i()->orientation );

		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}
	}

	/**
	 * Set the viewing method
	 *
	 * @return	void
	 */
	protected function setMethod()
	{
		$method = ( isset( \IPS\Request::i()->method ) ) ? \IPS\Request::i()->method : \IPS\Settings::i()->tutorials_index_type;
		
		\IPS\Request::i()->setCookie( 'tutorials_view', $method, ( new \IPS\DateTime )->add( new \DateInterval( 'P1Y' ) ) );
		
		if ( \IPS\Member::loggedIn()->member_id )
		{
			\IPS\Db::i()->replace( 'tutorials_view_method', array( 'member_id' => \IPS\Member::loggedIn()->member_id, 'method' => $method ) );
		}
		
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=index', 'front', 'tutorials' ) );
	}
}