<?php

namespace IPS\tutorials\modules\front\tutorials;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Submit Article Controller
 */
class _submit extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\tutorials\Article::canCreate( \IPS\Member::loggedIn(), NULL, TRUE );

		\IPS\Output::i()->sidebar['enabled'] = FALSE;

		parent::execute();
	}

	/**
	 * Choose category
	 *
	 * @return	void
	 */
	protected function manage()
	{
		/* Tutorialize a post */
		if( isset( \IPS\Request::i()->fromPost ) AND isset( \IPS\Request::i()->pid ) )
		{
			if ( !\IPS\Member::loggedIn()->group['g_tutorials_tutorializepost'] )
			{
				\IPS\Output::i()->error( 'no_permission_tutorialize', 'TUTORIALS-TUTORIALIZEPOSTPOST/1', 403, '' );
			}

			if( \IPS\Request::i()->fromPost == 1 AND \intval( \IPS\Request::i()->pid ) )
			{
				try
				{
					/* Check the Post ID and if the logged member has permission to view it */
					$post = \IPS\forums\Topic\Post::loadAndCheckPerms( \intval( \IPS\Request::i()->pid ) );
				}
				catch ( \OutOfRangeException $e )
				{
					\IPS\Output::i()->error( 'node_error_no_perm', 'TUTORIALS-TUTORIALIZEPOST/2', 403, '' );
				}
			}
		}

		$form = new \IPS\Helpers\Form( 'select_category', 'continue' );
		$form->class = 'ipsForm_vertical ipsForm_noLabels';
		$form->add( new \IPS\Helpers\Form\Node( 'tutcategory', isset( $data['category'] ) ? $data['category'] : NULL, TRUE, array(
			'url'					=> \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=submit', 'front', 'tutorials_submit' ),
			'class'					=> 'IPS\tutorials\Category',
			'permissionCheck'		=> 'add',
			'clubs'					=> \IPS\Settings::i()->club_nodes_in_apps
		) ) );

		if ( $values = $form->values() )
		{
			$url = \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=submit&do=submit', 'front', 'tutorials_submit' )->setQueryString( 'category', $values['tutcategory']->_id );

			if( isset( \IPS\Request::i()->_new ) )
			{
				$url = $url->setQueryString(array( '_new' => '1' ) );
			}

			/* Tutorialize a post */
			if( isset( \IPS\Request::i()->fromPost ) AND isset( \IPS\Request::i()->pid ) )
			{
				$url = $url->setQueryString( array( 'fromPost' => '1' ) );
				$url = $url->setQueryString( array( 'pid' => \IPS\Request::i()->pid ) );
			}
					
			\IPS\Output::i()->redirect( $url );
		}

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'select_category' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'submit' )->categorySelector( $form );
	}

	/**
	 * Submit tutorials
	 *
	 * @return	void
	 */
	protected function submit()
	{
		$category = NULL;

		/* Get category data */
		try
		{
			$category = \IPS\tutorials\Category::loadAndCheckPerms( \IPS\Request::i()->category );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=submit', 'front', 'tutorials_submit' ) );
		}

		$form = \IPS\tutorials\Article::create( $category );
		$form->class = 'ipsForm_vertical';

		$form->hiddenValues['category'] = $category->_id;
		$form->hiddenValues['postKey'] = ( \IPS\Request::i()->postKey ) ? \IPS\Request::i()->postKey : md5( uniqid() );
		$form->hiddenValues['csrfKey'] = \IPS\Session::i()->csrfKey;

		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'submit_new_tutorial' );
		\IPS\Output::i()->breadcrumb[] = array( $category->url(), $category->_title );

		$hasModOptions = false;

		if ( \IPS\tutorials\Article::modPermission( 'lock', NULL, $category ) or
			 \IPS\tutorials\Article::modPermission( 'pin', NULL, $category ) or
			 \IPS\tutorials\Article::modPermission( 'hide', NULL, $category ) or
			 \IPS\tutorials\Article::modPermission( 'feature', NULL, $category ) or
			 \IPS\Member::loggedIn()->group['g_tutorials_othermems'] )
		{
			$hasModOptions = TRUE;
		}

		if ( $club = $category->club() )
		{
			\IPS\core\FrontNavigation::$clubTabActive = TRUE;
			\IPS\Output::i()->breadcrumb = array();
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=clubs&controller=directory', 'front', 'clubs_list' ), \IPS\Member::loggedIn()->language()->addToStack('module__core_clubs') );
			\IPS\Output::i()->breadcrumb[] = array( $club->url(), $club->name );
			\IPS\Output::i()->breadcrumb[] = array( $category->url(), $category->_title );
			
			if ( \IPS\Settings::i()->clubs_header == 'sidebar' )
			{
				\IPS\Output::i()->sidebar['contextual'] = \IPS\Theme::i()->getTemplate( 'clubs', 'core' )->header( $club, $category, 'sidebar' );
			}
		}
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'submit_new_tutorial' ) );

		/* Online User Location */
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=submit', 'front', 'tutorials_submit' ), array(), 'loc_tutorials_adding_article' );

		/* Display */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'submit_new_tutorial' );

		//\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'submit' )->createArticle( $form->customTemplate( array( call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'submit', 'tutorials' ) ), 'createArticleForm' ), $hasModOptions ), $category, \IPS\Member::loggedIn()->language()->addToStack( 'submit_new_tutorial' ) );
		$formTemplate = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'submit', 'tutorials' ), 'createArticleForm' ), $category, $hasModOptions, NULL );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'submit', 'tutorials' )->createArticle( $formTemplate, $category, \IPS\Member::loggedIn()->language()->addToStack( 'submit_new_tutorial' ) );
	}
}