<?php

namespace IPS\tutorials\modules\front\tutorials;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _view extends \IPS\Content\Controller
{
	protected static $contentModel = 'IPS\tutorials\Article';

	public function execute()
	{
		try
		{
			$this->article = \IPS\tutorials\Article::loadAndCheckPerms( \IPS\Request::i()->id );

			if ( !$this->article->canView( \IPS\Member::loggedIn() ) )
			{
				\IPS\Output::i()->error( 'node_error', 'TUTORIALS-VIEW/1', 404, '' );
			}

			if ( $this->article->thumb )
			{
				\IPS\Output::i()->metaTags['og:image'] = \IPS\File::get( 'tutorials_Image', $this->article->thumb )->url;
			}
		}
        catch( \OutOfRangeException $e )
        {
			\IPS\Output::i()->error( 'node_error', 'TUTORIALS-VIEW/2', 404, '' );
		}

		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'tutorials.css', 'tutorials' ) );

		if( $this->article->image )
		{
			\IPS\Output::i()->metaTags['og:image'] = \IPS\File::get( 'tutorials_Image', $this->article->image )->url;
		}

		if( $this->article->videos )
		{
			$videos = $this->article->getVideos();
			foreach( $videos as $video )
			{
				\IPS\Output::i()->metaTags['og:video'] = $video['a_video_url'];
			}
		}

		parent::execute();
	}

	protected function manage()
	{
		parent::manage();

        $tabs = $this->article->commentReviewTabs();
        $_tabs = array_keys( $tabs );
        $tab = isset( \IPS\Request::i()->tab ) ? \IPS\Request::i()->tab : array_shift( $_tabs );
        $activeTabContents = $this->article->commentReviews( $tab );
        $commentsAndReviews = \count( $tabs ) ? \IPS\Theme::i()->getTemplate( 'global', 'core' )->commentsAndReviewsTabs( \IPS\Theme::i()->getTemplate( 'global', 'core' )->tabs( $tabs, $tab, $activeTabContents, $this->article->url(), 'tab', false, true ), md5( $this->article->url() ) ) : null;

		if( \IPS\Request::i()->isAjax() AND ( !isset( \IPS\Request::i()->preview ) OR !\IPS\Request::i()->preview ) )
		{
			\IPS\Output::i()->output = $activeTabContents;
            return;
		}

		if ( \IPS\Member::loggedIn()->member_id )
		{
			if ( \IPS\Settings::i()->tutorials_logvisit_author )
			{
				$save = array(
							'tutorial_id' 	=> $this->article->id,
							'member_id'		=> \IPS\Member::loggedIn()->member_id,
						    'date'          => time()
				);

				\IPS\Db::i()->insert( 'tutorials_articles_views', $save );
			}
			else
			{
				if ( \IPS\Member::loggedIn()->member_id != $this->article->mid )
				{
					$save = array(
								'tutorial_id' 	=> $this->article->id,
								'member_id'		=> \IPS\Member::loggedIn()->member_id,
							    'date'          => time()
					);
					
					\IPS\Db::i()->insert( 'tutorials_articles_views', $save );
				}
			}
		}
		
		if ( \IPS\Request::i()->isAjax() and \IPS\Request::i()->preview )
		{
			\IPS\Output::i()->sendOutput( \IPS\Theme::i()->getTemplate( 'browse' )->tutorialHover( $this->article ) );
			return;
		}

		\IPS\Session::i()->setLocation( $this->article->url(), $this->article->onlineListPermissions(), 'loc_tutorials_viewing_article', array( $this->article->name => FALSE ) );

        $cfields = [ ];
        $fields = $this->article->customFields();

		foreach( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( 'pfd.*', array( 'tutorials_cfields', 'pfd' ), NULL, 'pfd.cf_position' ), 'IPS\tutorials\Field' ) as $field )
		{
			if( array_key_exists( 'field_' . $field->id, $this->article->customFields() ) )
			{
				if ( $fields[ 'field_' . $field->id ] !== null AND $fields[ 'field_' . $field->id ] !== '' )
				{
					$cfields[ 'field_' . $field->id ] = array( 
						'type'		=> $field->type, 
						'key'		=> 'field_' . $field->id, 
						'value'		=> $field->displayValue( $fields[ 'field_' . $field->id ] ),
						'location'	=> $field->display_location
					);
				}
			}
		}

		$this->article->markRead();

		$videos = $this->article->getVideos();

		\IPS\Output::i()->sidebar['sticky'] = TRUE;
		\IPS\Output::i()->output .= \IPS\Theme::i()->getTemplate( 'view' )->view( $this->article, $commentsAndReviews, $this->article->nextItem(), $this->article->prevItem(), $cfields, $videos );
	}
	
	public function changeAuthor()
	{
		if ( !$this->article->canChangeAuthor() )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-CHANGEAUTHOR/1', 403, '' );
		}
		
		$form = new \IPS\Helpers\Form( 'change', 'change_tutorial_author_uc' );
		$form->class = 'ipsForm_vertical ipsPad';
		$form->add( new \IPS\Helpers\Form\Member( 'author', NULL, TRUE ) );

		if ( $values = $form->values() )
		{
			$this->article->changeAuthor( $values['author'] );

			$newAuthorPref  = json_decode( $values['author']->auto_track, true );
			if( $newAuthorPref['content'] == 1 )
			{
				$app	= 'tutorials';
				$area	= 'article';
				$id		= $this->article->id;
				$mid	= $values['author']->member_id;

				$count = \IPS\Db::i()->select( 'COUNT(*)', 'core_follow', array( 'follow_app=? AND follow_area=? AND follow_rel_id=? AND follow_member_id=?', $app, $area, $id, $mid ) )->first();
				if( $count == 0 )
				{
					$save = array(
						'follow_id'				=> md5( $app . ';' . $area . ';' . $id . ';' .  $mid ),
						'follow_app'			=> $app,
						'follow_area'			=> $area,
						'follow_rel_id'			=> $id,
						'follow_member_id'		=> $mid,
						'follow_is_anon'		=> 0,
						'follow_added'			=> time(),
						'follow_notify_do'		=> 1,
						'follow_notify_meta'	=> '',
						'follow_notify_freq'	=> 'immediate',
						'follow_notify_sent'	=> 0,
						'follow_visible'		=> 1
					);
			
					\IPS\Db::i()->insert( 'core_follow', $save );
				}
			}
			\IPS\Output::i()->redirect( $this->article->url() );
		}
		
		\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'forms', 'core' ), 'popupTemplate' ) );
	}

	protected function copy()
	{
		if( !$this->article->canMove() )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-COPYARTICLE/1', 403, '' );
		}

		$form = new \IPS\Helpers\Form( 'select_category', 'continue' );
		$form->class = 'ipsForm_vertical';
		$form->add( new \IPS\Helpers\Form\Member( 'tutauthor', $this->article->author(), TRUE ) );
		$form->add( new \IPS\Helpers\Form\Text( 'tuttitle', $this->article->name, TRUE ) );
		$form->add( new \IPS\Helpers\Form\Node( 'tutcategory', isset( $data['category'] ) ? $data['category'] : NULL, TRUE, array(
			'url'				=> \IPS\Http\Url::internal( 'app=tutorials&module=tutorials&controller=submit', 'front', 'tutorials_submit' ),
			'class'				=> 'IPS\tutorials\Category',
			'permissionCheck'	=> 'add',
			'clubs'				=> \IPS\Settings::i()->club_nodes_in_apps
		) ) );

		if( $values = $form->values() )
		{
			$article 			= \IPS\tutorials\Article::createItem( $values['tutauthor'], $values['tutauthor']->ip_address, \IPS\DateTime::ts( time() ), $values['tutcategory'], FALSE );
			$article->name		= $values['tuttitle'];
			$article->name_seo 	= \IPS\Http\Url\Friendly::seoTitle( $values['tuttitle'] );
			$article->content 	= $this->article->content;
			$article->desc 		= $this->article->desc;
			$article->featured	= $this->article->featured;
			$article->pinned	= $this->article->pinned;
			$article->locked	= $this->article->locked;
			$article->approved 	= $this->article->approved;

			if( $this->article->image )
			{
				$original = \IPS\File::get( 'tutorials_Image', $this->article->image );
				$image = \IPS\Image::create( $original->contents() );
				$article->image = \IPS\File::create( 'tutorials_Image', $original->filename, (string) $image );
			}

			$article->save();

			if( $this->article->image )
			{
				\IPS\Db::i()->update( 'tutorials_articles', array( 'a_thumb' => (string) $original->thumbnail( 'tutorials_Image' ) ), array( 'a_id=?', $article->id ) );
			}

			$article->setTags( $this->article->prefix() ? array_merge( $this->article->tags(), array( 'prefix' => $this->article->prefix() ) ) : $this->article->tags() );
			$article->save();

			foreach( \IPS\Db::i()->select( '*', 'core_content_meta', array( 'meta_item_id=? AND meta_class=? AND meta_type=?', $this->article->id, 'IPS\tutorials\Article', 'core_ContentMessages' ) ) AS $message )
			{
				$toInsert = array(
					'meta_class'	=> 'IPS\tutorials\Article',
					'meta_item_id'	=> $article->id,
					'meta_type'		=> 'core_ContentMessages',
					'meta_data'		=> $message['meta_data']
				);
				\IPS\Db::i()->insert( 'core_content_meta', $toInsert, TRUE );

				$article->meta_data	= 1;
				$article->save();
			}

			$toInsert 	= array();
			$i = 0;
			foreach( \IPS\Db::i()->select( '*', 'tutorials_articles_videos', array( 'a_video_tutorial_id=?', $this->article->id ) ) AS $video )
			{
				$toInsert = array(
					'a_video_member_id'		=> $video['a_video_member_id'],
					'a_video_tutorial_id'	=> $article->id,
					'a_video_title'			=> $video['a_video_title'],
					'a_video_content'		=> $video['a_video_content'],
					'a_video_url'			=> $video['a_video_url'],
					'a_video_embed'			=> $video['a_video_embed'],
					'a_video_date'			=> $video['a_video_date'],
				);
				\IPS\Db::i()->insert( 'tutorials_articles_videos', $toInsert, TRUE );
				$i++;
			}
			$article->videos = $i;
			$article->save();

			\IPS\Output::i()->redirect( $article->url(), 'saved' );
		}

		\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'forms', 'core' ), 'popupTemplate' ) );
	}

	protected function read()
	{
		if ( !$this->article->canViewReaders() )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-VIEWREADERS/1', 403, '' );
		}

		$table = new \IPS\Helpers\Table\Db( 'tutorials_articles_views', $this->article->url()->setQueryString( 'do', 'read' ), array( 'tutorial_id=?', $this->article->id ) );
		$table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'view' ), 'logTable' );
		$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'view' ), 'logRows' );
		$table->sortBy = 'date';
		$table->limit = 10;

		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'view' )->log( $this->article, (string) $table );
	}

	protected function image()
	{
		if( !$this->article->canEdit() OR !$this->article->hasAttachments() )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-SETIMAGE/1', 403, '' );
		}

		foreach( \IPS\Db::i()->select( '*', 'core_attachments_map', array( 'id1=? AND location_key=? AND attach_is_image=?', $this->article->id, 'tutorials_Article', 1 ) )->join( 'core_attachments', 'core_attachments_map.attachment_id=core_attachments.attach_id' ) as $attachment )
		{
			$attachments[] = $attachment['attachment_id'];
		}

		$table = new \IPS\Helpers\Table\Db( 'core_attachments', $this->article->url()->setQueryString( 'do', 'image' ), \IPS\Db::i()->in( 'attach_id', $attachments ) );
		$table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'view' ), 'imagesTable' );
		$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'view' ), 'imagesRows' );
		$table->sortBy = 'attach_id';
		$table->sortDirection	= 'asc';
		$table->limit = 10;

		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'view' )->images( $this->article, (string) $table );
	}

	protected function useImage()
	{
		$id 		= \intval( \IPS\Request::i()->imgid );
		$attachment = \IPS\Db::i()->select( '*', 'core_attachments', array( 'attach_id=?', $id ) )->first();
		$file		= \IPS\File::get( 'core_Attachment', $attachment['attach_location'] );

		$this->article->image = (string) $file;
		$this->article->save();

		try
		{
			$original = \IPS\File::get( 'tutorials_Image', $this->article->image );
			\IPS\Db::i()->update( 'tutorials_articles', array( 'a_thumb' => (string) $original->thumbnail( 'tutorials_Image' ) ), array( 'a_id=?', $this->article->id ) );
		}
		catch( \Exception $e ){}

		\IPS\Output::i()->redirect( $this->article->url(), 'saved' );
	}

	protected function changeDate()
	{
		if ( !$this->article->canChangeTutorialDate() )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-CHANGEDATE/1', 403, '' );
		}

		$date = '';
		if( $this->article->last_comment )
		{
			$date = $this->article->last_comment;
		}
		if( $this->article->last_review AND $this->article->last_review > $this->article->last_comment )
		{
			$date = $this->article->last_review;
		}

		$form = new \IPS\Helpers\Form;

		if( $date )
		{
			$form->add( new \IPS\Helpers\Form\Date( 'tutorial_new_date', $this->article->date, FALSE, array( 'time' => TRUE, 'max' => \IPS\DateTime::ts( $date ) ), NULL, NULL, NULL, 'tutorial_new_date' ) );
		}
		else
		{
			$form->add( new \IPS\Helpers\Form\Date( 'tutorial_new_date', $this->article->date, FALSE, array( 'time' => TRUE ), NULL, NULL, NULL, 'tutorial_new_date' ) );
		}

		if ( $values = $form->values() )
		{
			$this->article->date = $values['tutorial_new_date']->getTimestamp();
			$this->article->save();

			\IPS\Content\Search\Index::i()->removeFromSearchIndex( $this->article );
			\IPS\Content\Search\Index::i()->index( $this->article );

			\IPS\Session::i()->modLog( 'modlog__tutorials_change_date', array( $this->article->url()->__toString() => FALSE, $this->article->name => FALSE ), $this->article );

			\IPS\Output::i()->redirect( $this->article->url(), 'saved' );
		}

		\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'forms', 'core' ), 'popupTemplate' ) );
	}

	protected function videoForm()
	{
		if( !$this->article->container()->allow_videos OR !$this->article->canEdit() )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-VIDEO FORM/1', 403, '' );
		}

		$form  = new \IPS\Helpers\Form;
		$video = array();
		$form->hiddenValues['type'] = 'add';

		if( \IPS\Request::i()->type == 'editVideo' )
		{
			$videoId = \IPS\Request::i()->vid;
			if( !$videoId )
			{
				\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-ADDVIDEOURL/2', 403, '' );
			}

			$video = \IPS\Db::i()->select( '*', 'tutorials_articles_videos', array( 'a_video_id=?', $videoId ) )->first();
			$form->hiddenValues['type'] = 'edit';
		}

		$form->add( new \IPS\Helpers\Form\Text( 'a_video_title', \count( $video ) ? $video['a_video_title'] : NULL, TRUE ) );
		$form->add( new \IPS\Helpers\Form\Url( 'a_video_url', \count( $video ) ? $video['a_video_url'] : NULL, TRUE, array( 'allowedProtocols' => array( 'http', 'https' ) ), NULL, NULL, NULL, 'tutorial_videos_url' ) );
		$form->add( new \IPS\Helpers\Form\Editor( 'a_video_content', isset( $video['a_video_content'] ) ? $video['a_video_content'] : NULL, FALSE, array(
			'app'			=> 'tutorials',
			'key'			=> 'Tutorials',
			'autoSaveKey'	=> ( \IPS\Request::i()->type == 'editVideo' ? "tutorials-video-{$videoId}" : "tutorials-new-video" )
		) ) );

		if( $values = $form->values() )
		{
			$videoHost 	= parse_url( $values['a_video_url'] );

			switch( $videoHost['host'] )
			{
				case 'vimeo.com':
					$embed = 'https://player.vimeo.com/video/'. $this->article->getVideoId( $values['a_video_url'] );
				break;
				case 'www.youtube.com';
					$embed = 'https://www.youtube.com/embed/'. $this->article->getVideoId( $values['a_video_url'] );
				break;
			}

			if( $values['type'] == 'add' )
			{
				$toInsert = array(
					'a_video_title'			=> $values['a_video_title'],
					'a_video_url'			=> $values['a_video_url'],
					'a_video_embed'			=> $embed,
					'a_video_content'		=> $values['a_video_content'],
					'a_video_member_id'		=> \IPS\Member::loggedIn()->member_id,
					'a_video_tutorial_id'	=> $this->article->id,
					'a_video_date'			=> time()
				);
				$id = \IPS\Db::i()->insert( 'tutorials_articles_videos', $toInsert, TRUE );

				/* Update Tutorials Videos */
				$this->article->videos += 1;
				$this->article->save();
			}
			else
			{
				$toUpdate = array(
					'a_video_title'			=> $values['a_video_title'],
					'a_video_url'			=> $values['a_video_url'],
					'a_video_embed'			=> $embed,
					'a_video_content'		=> $values['a_video_content'],
				);
				\IPS\Db::i()->update( 'tutorials_articles_videos', $toUpdate, array( 'a_video_id=?', $video['a_video_id'] ) );
				$id = $video['a_video_id'];
			}

			\IPS\Output::i()->redirect( $this->article->url()->setQueryString( array( 'do' => 'viewVideo', 'vid' => $id ) ), 'saved' );
		}

		if( !\IPS\Request::i()->isAjax() )
		{
			foreach ( $this->article->container()->parents() as $parent )
			{
				\IPS\Output::i()->breadcrumb[] = array( $parent->url(), $parent->_title );
			}
			\IPS\Output::i()->breadcrumb[] = array( $this->article->container()->url(), $this->article->container()->_title );
			\IPS\Output::i()->breadcrumb[] = array( $this->article->url(), $this->article->name );
			\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_videos') );

			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_videos') . ' - ' . $this->article->name;
		}

		\IPS\Output::i()->output = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'forms', 'core' ), 'popupTemplate' ) );
	}

	protected function viewVideo()
	{
		$vid = \intval( \IPS\Request::i()->vid );
		if ( !$vid )
		{
			\IPS\Output::i()->error( 'node_error', 'TUTORIALS-VIEW VIDEO/1', 404, '' );
		}

		$video = $this->article->getVideo( $vid );

		foreach( $this->article->container()->parents() AS $parent )
		{
			\IPS\Output::i()->breadcrumb[] = array( $parent->url(), $parent->_title );
		}
		\IPS\Output::i()->breadcrumb[] = array( $this->article->container()->url(), $this->article->container()->_title );
		\IPS\Output::i()->breadcrumb[] = array( $this->article->url(), \IPS\Member::loggedIn()->language()->addToStack( $this->article->name ) );
		$msg = \IPS\Member::loggedIn()->language()->addToStack( 'a_video_viewing', FALSE, array( 'sprintf' => array( $video['a_video_title'] ) ) );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $msg );

		\IPS\Output::i()->title = $msg;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'view' )->viewVideo( $this->article, $video );
	}

	protected function deleteVideo()
	{
		\IPS\Session::i()->csrfKey;

		if( !$this->article->container()->allow_videos OR !$this->article->canEdit() )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-DELETE VIDEO/1', 403, '' );
		}

		\IPS\Db::i()->delete( 'tutorials_articles_videos', array( 'a_video_id=?', \IPS\Request::i()->vid ) );

		$this->article->videos -= 1;
		$this->article->save();

		\IPS\Output::i()->redirect( $this->article->url(), 'deleted' );
	}

	public function edit()
	{
		try
		{
			$class		= static::$contentModel;
			$article	= $class::loadAndCheckPerms( \IPS\Request::i()->id );
			$container 	= $article->container();
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS-EDIT/1', 403, 'no_module_permission_guest' );
		}

		if ( isset( \IPS\Request::i()->form_submitted ) )
		{
			if( !$article->couldEdit() )
			{
				\IPS\Output::i()->error( 'edit_no_perm_err', 'TUTORIALS-EDIT/2', 403, '' );
			}
		}
		else
		{
			if ( ! $article->canEdit() )
			{
				\IPS\Output::i()->error( 'edit_no_perm_err', 'TUTORIALS-EDIT/3', 403, '' );
			}
		}

		$formElements 	= $class::formElements( $article, $container );
		$hasModOptions 	= FALSE;
		$form 			= $article->buildEditForm();

		if ( $values = $form->values() )
		{
			if ( $article->canEdit() )
			{				
				$article->processForm( $values );
				$article->save();
				$article->processAfterEdit( $values );

				\IPS\Session::i()->modLog( 'modlog__item_edit', array( $article::$title => FALSE, $article->url()->__toString() => FALSE, $article::$title => TRUE, $article->mapped( 'title' ) => FALSE ), $article );

				\IPS\Output::i()->redirect( $article->url() );
			}
			else
			{
				$form->error = \IPS\Member::loggedIn()->language()->addToStack('edit_no_perm_err');
			}
		}

		\IPS\Output::i()->sidebar['enabled'] = FALSE;

		$formTemplate = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'submit', 'tutorials' ), 'createArticleForm' ), $container, $hasModOptions, $article );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'submit', 'tutorials' )->createArticle( $formTemplate, $container, \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_edit_article' ) );

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_edit_article' ) );
		\IPS\Output::i()->breadcrumb[] = array( $article->url(), $article->mapped('title') );
		\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_edit_article' ) );
	}

	public function convertToComment()
	{
		\IPS\Session::i()->csrfCheck();

		if( ! $this->article->container()->allow_comments )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'LINKS DIRECTORY - CONVERT REVIEW TO COMMENT/1', 403, '' );
		}

		if( ! \IPS\Member::loggedIn()->modPermission( 'can_convert_tutorial_rev_com' ) )
		{
			\IPS\Output::i()->error( 'no_module_permission', 'TUTORIALS - CONVERT REVIEW TO COMMENT/1', 403, '' );
		}

		try
		{
			$review = \IPS\tutorials\Article\Review::loadAndCheckPerms( \IPS\Request::i()->review );
		}
		catch( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', 'TUTORIALS - CONVERT REVIEW TO COMMENT/2', 403 );
		}

		$comment = \IPS\tutorials\Article\Comment::create( $this->article, $review->text, FALSE, NULL, NULL, \IPS\Member::load( $review->mid ), \IPS\DateTime::ts( $review->date ), $review->ip );

		$review->delete();

		\IPS\Session::i()->modLog( 'modlog__tutorials_convert_rev_com', array( $this->article->url()->setQueryString( array( 'do' => 'findComment', 'comment' => $comment->id ) )->__toString() => FALSE, $this->article->name => FALSE ), $this->article );

		\IPS\Output::i()->redirect( $this->article->url()->setQueryString( array( 'do' => 'findComment', 'comment' => $comment->id ) ), 'completed' );
	}
}