<?php


namespace IPS\tutorials\setup\upg_200000;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * 2.0.0 Upgrade Code
 */
class _Upgrade
{
	/**
	 * ...
	 *
	 * @return	array	If returns TRUE, upgrader will proceed to next step. If it returns any other value, it will set this as the value of the 'extra' GET parameter and rerun this step (useful for loops)
	 */
	public function step1()
	{
		foreach( \IPS\Db::i()->select( '*', 'tutorials_cats' )->join( 'core_permission_index', array( 'app=? and perm_type=? and perm_type_id=tutorials_cats.c_id', 'tutorials', 'cat' ) ) as $category )
		{
			$cid		= $category['c_id'];
			$name		= $category['c_name'];
			$desc		= $category['c_desc'];
			$allow_tag	= $category['c_allow_tags'];
			$allow_pref	= $category['c_allow_prefixes'];
			$sort		= $category['c_artorder'];

			switch( $sort )
			{
				case 'update';
					$c_artorder = 'a_updated';
				break;
				case 'views':
					$c_artorder = 'a_views';
				break;
				case 'author':
					$c_artorder = 'a_mid';
				break;
				case 'date':
					$c_artorder = 'a_date';
				break;
				case 'title':
					$c_artorder = 'a_name';
				break;
			}

			$category	= \IPS\tutorials\Category::constructFromData( $category );

			\IPS\Lang::saveCustom( 'tutorials', "tutorials_category_{$cid}", trim( $name ) );
			\IPS\Lang::saveCustom( 'tutorials', "tutorials_category_{$cid}_desc", \IPS\Text\Parser::parseStatic( $desc, TRUE, NULL, NULL, TRUE, TRUE, TRUE ) );

			try
			{
				$latestArticle	= \IPS\Db::i()->select( '*', 'tutorials_articles', array( 'a_approved=? and a_cat=?', 1, $cid ), 'a_date DESC', array( 0, 1 ) )->first();

				$category->last_id		= $latestArticle['a_id'];
				$category->last_time	= $latestArticle['a_date'];
			}
			catch( \UnderflowException $e )
			{
				$category->last_id		= 0;
				$category->last_time	= 0;
			}

			$category->article_moderation = 0;
			$category->tags_disabled 	  = $allow_tag  == 1 ? 0 : 1;
			$category->tags_noprefixes 	  = $allow_pref == 1 ? 0 : 1;
			$category->artorder 		  = $c_artorder;

			$category->save();
		}

		/* Fix the PARENT category */
		\IPS\Db::i()->changeColumn( 'tutorials_cats', 'c_parent', array(
			'name'			=> 'c_parent',
			'type'			=> 'int',
			'length'		=> 10,
			'allow_null'	=> false,
			'default'		=> 0,
			'auto_increment'=> false
		) );

		\IPS\Db::i()->createTable( array(
			'name'		=> 'tutorials_articles_views',
			'columns'	=> array(
				array(
					'name'				=> 'id',
					'type'				=> 'BIGINT',
					'length'			=> 20,
					'allow_null'		=> false,
					'auto_increment'	=> true
				),
				array(
					'name'			=> 'tutorial_id',
					'type'			=> 'INT',
					'length'		=> 11,
					'allow_null'	=> false,
					'default'		=> 0
				),
				array(
					'name'			=> 'member_id',
					'type'			=> 'MEDIUMINT',
					'length'		=> 8,
					'allow_null'	=> false,
					'default'		=> 0
				),
				array(
					'name'			=> 'date',
					'type'			=> 'INT',
					'length'		=> 10,
					'allow_null'	=> false,
					'default'		=> 0
				),
			),
			'indexes'	=> array(
				array(
					'type'		=> 'primary',
					'columns'	=> array( 'id' )
				),
			)
		) );

		\IPS\Db::i()->update( 'core_permission_index', array( 'perm_type' => 'category' ), array( 'app=?', 'tutorials' ) );

		\IPS\Db::i()->dropColumn( 'tutorials_cats', array( 'c_name' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_cats', array( 'c_desc' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_cats', array( 'c_can_rss' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_cats', array( 'c_allow_articles' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_cats', array( 'c_allow_prefixes' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_cats', array( 'c_allow_tags' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_articles', array( 'a_notified' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_articles', array( 'a_post_key' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_articles', array( 'a_hasattach' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_articles', array( 'a_votes' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_articles', array( 'a_show_sig' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_articles', array( 'a_comments_queued' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_articles', array( 'a_deleted' ) );
		\IPS\Db::i()->dropColumn( 'tutorials_articles', array( 'a_date_deleted' ) );

		\IPS\Db::i()->dropTable( 'tutorials_mods', TRUE );

		return true;
	}
	
	/**
	 * Custom title for this step
	 *
	 * @return string
	 */
	public function step1CustomTitle()
	{
		return "Upgrading Tutorials Categories";
	}

	/**
	 * Custom field translations
	 *
	 * @return	array	If returns TRUE, upgrader will proceed to next step. If it returns any other value, it will set this as the value of the 'extra' GET parameter and rerun this step (useful for loops)
	 */
	public function step2()
	{
		foreach( \IPS\Db::i()->select( '*', 'tutorials_cfields' ) as $field )
		{
			\IPS\Lang::saveCustom( 'core', "tutorials_field_{$field['cf_id']}", $field['cf_title'] );
			\IPS\Lang::saveCustom( 'core', "tutorials_field_{$field['cf_id']}_desc", $field['cf_desc'] );

			$update	= array();

			switch( $field['cf_type'] )
			{
				case 'input':
					$update['cf_type']	= 'Text';
				break;

				case 'textarea':
					$update['cf_type']	= 'TextArea';
				break;

				case 'drop':
					$update['cf_type']	= 'Select';
				break;

				case 'radio':
					$update['cf_type']	= 'Radio';
				break;

				case 'cbox':
					$update['cf_type']		= 'Select';
					$update['cf_multiple']	= TRUE;
				break;

				default:
					$update['cf_type']	= 'Text';
				break;
			}

			if( $field['cf_type'] === 'Textarea' )
			{
				$update['cf_type']	= 'TextArea';
			}

			/* Fix the options */
			if( $field['cf_content'] )
			{
				$options	= explode( '|', $field['cf_content'] );
				$newOptions	= array();

				foreach( $options as $option )
				{
					list( $k, $v )	= explode( '=', $option );
					$newOptions[]	= $v;
				}

				if( !isset( $_SESSION['pfieldsd'] ) )
				{
					$_SESSION['pfieldsd']	= array();
				}

				$_SESSION['pfieldsd'][ $field['cf_id'] ]	= $field['cf_id'];
				$update['cf_content']	= json_encode( $newOptions );
			}

			if( \count( $update ) )
			{
				\IPS\Db::i()->update( 'tutorials_cfields', $update, 'cf_id=' . $field['cf_id'] );
			}

			/* Indexes are needed, otherwise you get an error if you try to edit the custom field later */
			\IPS\Db::i()->addIndex( 'tutorials_ccontent', array( 'name' => 'field_' . $field['cf_id'], 'type' => 'key', 'columns' => array( 'field_' . $field['cf_id'] ), 'length' => array( 255 ) ) );
		}

		\IPS\Db::i()->dropColumn( 'tutorials_cfields', array( 'cf_title', 'cf_desc' ) );

		return TRUE;
	}

	/**
	 * Custom title for this step
	 *
	 * @return string
	 */
	public function step2CustomTitle()
	{
		return "Upgrading Tutorials Custom Fields";
	}

	/**
	 * Finish - This is run after all apps have been upgraded
	 *
	 * @return	mixed	If returns TRUE, upgrader will proceed to next step. If it returns any other value, it will set this as the value of the 'extra' GET parameter and rerun this step (useful for loops)
	 * @note	We opted not to let users run this immediately during the upgrade because of potential issues (it taking a long time and users stopping it or getting frustrated) but we can revisit later
	 */
	public function finish()
    {
	   \IPS\Task::queue( 'core', 'RebuildPosts', array( 'class' => 'IPS\tutorials\Article' ), 2 );
	   \IPS\Task::queue( 'core', 'RebuildPosts', array( 'class' => 'IPS\tutorials\Article\Comment' ), 2 );
	   \IPS\Task::queue( 'core', 'RebuildNonContentPosts', array( 'extension' => 'tutorials_Categories' ), 2 );

        return TRUE;
    }
}