<?php

namespace IPS\tutorials;

/* To prevent PHP errors (extending class does not exist) revealing path */
if( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _Article extends \IPS\Content\Item implements
\IPS\Content\Permissions,
\IPS\Content\Tags,
\IPS\Content\Followable,
\IPS\Content\ReadMarkers,
\IPS\Content\Views,
\IPS\Content\Hideable,
\IPS\Content\Featurable,
\IPS\Content\Pinnable,
\IPS\Content\Lockable,
\IPS\Content\Shareable,
\IPS\Content\Ratings,
\IPS\Content\Searchable,
\IPS\Content\Embeddable,
\IPS\Content\MetaData
{
	use \IPS\Content\Reactable, \IPS\Content\Reportable, \IPS\Content\ItemTopic;

	public static $application = 'tutorials';
	public static $module = 'tutorials';
	public static $databaseTable = 'tutorials_articles';
	public static $databasePrefix = 'a_';
	protected static $multitons;
	public static $containerNodeClass = 'IPS\tutorials\Category';
	public static $commentClass = 'IPS\tutorials\Article\Comment';
	public static $reviewClass = 'IPS\tutorials\Article\Review';

	public static $databaseColumnMap = array(
		'container'				=> 'cat',
		'author'				=> 'mid',
		'views'					=> 'views',
		'title'					=> 'name',
		'content'				=> 'content',
		'num_comments'			=> 'comments',
		'unapproved_comments'	=> 'unapproved_comments',
		'hidden_comments'		=> 'hidden_comments',
		'last_comment'			=> 'last_comment',
		'last_review'			=> 'last_review',
		'date'					=> 'date',
		'updated'				=> 'updated',
		'rating'				=> 'rating',
		'approved'				=> 'approved',
		'approved_by'			=> 'approver',
		'approved_date'			=> 'approvedon',
		'pinned'				=> 'pinned',
		'featured'				=> 'featured',
		'locked'				=> 'locked',
		'ip_address'			=> 'ipaddress',
		'meta_data'				=> 'meta_data',
		'num_reviews'			=> 'reviews',
		'unapproved_reviews'	=> 'unapproved_reviews',
		'hidden_reviews'		=> 'hidden_reviews',
	);

	public static $title = 'tutorials_article';
	public static $icon = 'comments';
	public $tableHoverUrl = TRUE;
	public static $formLangPrefix = 'a_';
	public static $reputationType = 'a_id';
	public static $hideLogKey = 'tutorials-article';
	public static $ownerTypes = array( 'member' => 'a_mid' );

	public static function basicDataColumns()
	{
		$return = parent::basicDataColumns();
		$return[] = 'a_thumb';
		$return[] = 'a_image';
		return $return;
	}

	public function set_name( $name )
	{
		$this->_data['name'] = $name;
		$this->_data['name_seo'] = \IPS\Http\Url::seoTitle( $name );
	}

	public function get_name_seo()
	{
		if( !$this->_data['name_seo'] )
		{
			$this->name_seo	= \IPS\Http\Url::seoTitle( $this->name );
			$this->save();
		}

		return $this->_data['name_seo'] ?: \IPS\Http\Url::seoTitle( $this->name );
	}

	public static $urlBase = 'app=tutorials&module=tutorials&controller=view&id=';
	public static $urlTemplate = 'tutorials_article';
	public static $seoTitleColumn = 'name_seo';
	protected $_url	= array();

	public function lastCommentPageUrl()
	{
		return parent::lastCommentPageUrl()->setQueryString( 'tab', 'comments' );
	}

	public function lastReviewPageUrl()
	{
		return parent::lastCommentPageUrl()->setQueryString( 'tab', 'reviews' );
	}

	public static function contentTableTemplate()
	{
		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'tutorials.css', 'tutorials' ) );
		return array( \IPS\Theme::i()->getTemplate( 'browse', 'tutorials' ), 'rows' );
	}

	protected $_customFields = NULL;
	protected $_fieldData = NULL;

	public function customFields( $topic = FALSE )
	{
		$return = array();
		$fields = $this->container()->cfields;

		if( $topic === TRUE AND $this->_fieldData === NULL )
		{
			$this->_fieldData	= iterator_to_array( \IPS\Db::i()->select( '*', 'tutorials_cfields', array( 'cf_topic=?', 1 ) )->setKeyField( 'cf_id' ) );
		}

		try
		{
			if( $this->_customFields === NULL )
			{
				$this->_customFields = \IPS\Db::i()->select( '*', 'tutorials_ccontent', array( 'art_id=?', $this->id ) )->first();
			}
			
			foreach( $this->_customFields as $k => $v )
			{
				$fieldId = str_replace( 'field_', '', $k );

				/* If we're getting fields for the topic we need to skip any that are set to not be included */
				if( $topic === TRUE and !isset( $this->_fieldData[ $fieldId ] ) )
				{
					continue;
				}

				if( array_key_exists( $fieldId, $fields ) )
				{
					if( $topic === TRUE )
					{
						$thisField = \IPS\tutorials\Field::constructFromData( $this->_fieldData[ $fieldId ] );

						if( isset( $this->_fieldData[ $fieldId ] ) )
						{
							$v	= str_replace( '{content}', htmlspecialchars( $v, ENT_DISALLOWED, 'UTF-8', FALSE ), $thisField->displayValue( $v ) );
							$v	= str_replace( '{member_id}', \IPS\Member::loggedIn()->member_id, $v );
							$v	= str_replace( '{title}', \IPS\Member::loggedIn()->language()->addToStack( 'tutorials_field_' . $fieldId ), $v );
						}
						else
						{
							$v	= htmlspecialchars( $v, ENT_DISALLOWED, 'UTF-8', FALSE );
						}
					}

					$return[ $k ] = $v;
				}
			}
		}
		catch( \UnderflowException $e ){}
		
		return $return;
	}
	
	public function commentReviewTabs()
	{
		$tabs = array();
		if( $this->container()->allow_comments )
		{
			$tabs['comments'] = \IPS\Member::loggedIn()->language()->pluralize( \IPS\Member::loggedIn()->language()->get( 'tutorial_comment_count' ), array( $this->mapped('num_comments') ) );
		}
		if( $this->container()->allow_reviews )
		{
			$tabs['reviews'] = \IPS\Member::loggedIn()->language()->pluralize( \IPS\Member::loggedIn()->language()->get( 'tutorial_review_count' ), array( $this->mapped('num_reviews') ) );
		}
			
		return $tabs;
	}

	public function canReview( $member = NULL, $considerPostBeforeRegistering = true )
	{
		return parent::canReview( $member, $considerPostBeforeRegistering );
	}

	public function commentReviews( $tab )
	{
		if( $tab === 'reviews' )
		{
			return \IPS\Theme::i()->getTemplate('view')->reviews( $this );
		}
		elseif( $tab === 'comments' )
		{
			return \IPS\Theme::i()->getTemplate('view')->comments( $this );
		}
		
		return '';
	}
	
	public static function moderateNewItems( \IPS\Member $member, \IPS\Node\Model $container = NULL, $considerPostBeforeRegistering = false )
	{
		if( $container and $container->article_moderation and !$member->group['g_avoid_q'] and !$container->can('post') )
		{
			return TRUE;
		}

		return parent::moderateNewItems( $member, $container, $considerPostBeforeRegistering );
	}
	
	public function moderateNewComments( \IPS\Member $member, $considerPostBeforeRegistering = false )
	{
		$commentClass = static::$commentClass;

		return ( $this->container()->comments_moderation and !$member->group['g_avoid_q'] ) or parent::moderateNewComments( $member );
	}

	public function moderateNewReviews( \IPS\Member $member, $considerPostBeforeRegistering = false )
	{
		return ( $this->container()->moderate_reviews and !$member->group['g_avoid_q'] ) or parent::moderateNewReviews( $member );
	}

	public static function getCommentsPerPage()
	{
		return \IPS\Settings::i()->tutorials_com_pp;
	}

	public function setDefaultValues()
	{
		static::$reviewsPerPage = \IPS\Settings::i()->tutorials_rev_pp;
	}

	public function canView( $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();

		if( !$this->container()->open and !$member->isAdmin() )
		{
			return FALSE;
		}
		return parent::canView( $member );
	}

	public function canEdit( $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		return ( $member->member_id == $this->author()->member_id ) or parent::canEdit( $member );
	}
	
	public static function getItemsWithPermission( $where=array(), $order=NULL, $limit=10, $permissionKey='read', $includeHiddenItems=NULL, $queryFlags=0, \IPS\Member $member=NULL, $joinContainer=FALSE, $joinComments=FALSE, $joinReviews=FALSE, $countOnly=FALSE, $joins=NULL, $skipPermission=FALSE, $joinTags=TRUE, $joinAuthor=TRUE, $joinLastCommenter=TRUE, $showMovedLinks=FALSE )
	{
		$member = $member ?: \IPS\Member::loggedIn();

		if( !$member->isAdmin() )
		{
			$where[] = array( 'tutorials_cats.c_open=1' );
			$joinContainer = TRUE;
		}

		if( \IPS\Settings::i()->tutorials_filterbyletters AND \IPS\Dispatcher::hasInstance() AND \IPS\Dispatcher::i()->application->directory === 'tutorials' and \IPS\Dispatcher::i()->module->key === 'tutorials' and \IPS\Dispatcher::i()->controller == 'browse' AND isset( \IPS\Request::i()->letter ) )
		{
			if( \IPS\Request::i()->letter != '0' )
			{
				$where[] = array( "LOWER( tutorials_articles.a_name ) LIKE CONCAT( ?, '%' )", mb_strtolower( trim( \IPS\Request::i()->letter ) ) );
			}
			else
			{
				foreach( range( 65, 90 ) as $letter )
				{
					$where[] = array( "LOWER( tutorials_articles.a_name ) NOT LIKE CONCAT( ?, '%' )", mb_strtolower( trim( chr( $letter ) ) ) );	
				}
			}
		}
				
		return parent::getItemsWithPermission( $where, $order, $limit, $permissionKey, $includeHiddenItems, $queryFlags, $member, $joinContainer, $joinComments, $joinReviews, $countOnly, $joins, $skipPermission, $joinTags, $joinAuthor, $joinLastCommenter, $showMovedLinks );
	}

	public static function canCreate( \IPS\Member $member, \IPS\Node\Model $container=NULL, $showError=FALSE )
	{
		$data = \IPS\tutorials\Members::getMemberData( $member->member_id );
		if( $member->member_id AND !$data['block_submissions'] )
		{
			if( $showError )
			{
				\IPS\Output::i()->error( 'tutorials_err_submissions_blocked', 'TUTORIALS - SUBMIT TUTORIAL/1', 403, '' );
			}
			
			return FALSE;
		}

		return parent::canCreate( $member, $container, $showError );
	}

	public static function formElements( $item=NULL, \IPS\Node\Model $container=NULL )
	{
		$title	 = NULL;
		$desc	 = NULL;
		$content = NULL;

		if( isset( \IPS\Request::i()->fromPost ) AND isset( \IPS\Request::i()->pid ) )
		{
			if( \IPS\Request::i()->fromPost == 1 AND \intval( \IPS\Request::i()->pid ) )
			{
				try
				{
					$post 	 = \IPS\forums\Topic\Post::loadAndCheckPerms( \intval( \IPS\Request::i()->pid ) );
					$title	 = $post->item()->mapped('title');
					$content = $post->post;
				}
				catch ( \OutOfRangeException $e )
				{
					\IPS\Output::i()->error( 'node_error_no_perm', 'TUTORIALS-TUTORIALIZEPOST-S/1', 403, '' );
				}
			}
		}

		$return = parent::formElements( $item, $container );
 
		if( $title !== NULL )
		{
			$def_title = $title;
		}
		else
		{
			if( $container->def_content )
			{
				$def_title = $container->def_title;
			}
			else
			{
				$def_title = '';
			}
		}

		$return['name']		= new \IPS\Helpers\Form\Text( "a_title", $item ? $item->name : $def_title, TRUE, array( 'maxLength' => 255 ) );
		$return['name']->rowClasses[] = 'ipsFieldRow_primary';
		$return['name']->rowClasses[] = 'ipsFieldRow_fullWidth';

		if( $desc !== NULL )
		{
			$def_desc = $desc;
		}
		else
		{
			if( $container->def_content )
			{
				$def_desc = $container->def_desc;
			}
			else
			{
				$def_desc = '';
			}
		}

		$return['desc']		= new \IPS\Helpers\Form\Textarea( 'a_desc', $item ? $item->desc : $def_desc, $container->desc_required ? TRUE: FALSE, array( 'maxLength' => \IPS\Settings::i()->tutorials_maxpreview ) );

		if( $content !== NULL )
		{
			$def_content = $content;
		}
		else
		{
			if( $container->def_content )
			{
				$def_content = $container->def_content;
			}
			else
			{
				$def_content = '';
			}
		}

		$return['content'] = new \IPS\Helpers\Form\Editor( 'a_content', $item ? $item->content : $def_content, TRUE, array( 'app' => 'tutorials', 'key' => 'Article', 'autoSaveKey' => 'tutorials-new-article', 'attachIds' => ( $item === NULL ? NULL : array( $item->id ) ) ) );

		$customFieldValues = $item ? $item->customFields() : array();
		foreach( $container->cfields as $k => $field )
		{
			$return[] = $field->buildHelper( isset( $customFieldValues[ "field_{$k}" ] ) ? $customFieldValues[ "field_{$k}" ] : NULL );
		}

		if( \IPS\Request::i()->do == 'submit' )
		{
	        if( \IPS\Member::loggedIn()->group['g_tutorials_othermems'] )
			{
				$return['author'] = new \IPS\Helpers\Form\Member( 'tutorial_author', NULL, FALSE, array(), function( $member )
		        {
					if( !$member )
					{
						$member = \IPS\Member::loggedIn();
					}

		            if( !\is_object( $member ) )
		            {
		                throw new \InvalidArgumentException( 'tutorial_invalid_member' );
		            }
		        },
		        NULL, NULL, 'tutorial_author' );
	    	}

			$options = array();
			$toggles = array();

			if( static::modPermission( 'lock', NULL, $container ) )
			{
				$options['lock'] = 'create_tutorial_locked';
				$toggles['lock'] = array( 'create_tutorial_locked' );
			}

			if( static::modPermission( 'pin', NULL, $container ) )
			{
				$options['pin'] = 'create_tutorial_pinned';
				$toggles['pin'] = array( 'create_tutorial_pinned' );
			}

			if( static::modPermission( 'hide', NULL, $container ) )
			{
				$options['hide'] = 'create_tutorial_hidden';
				$toggles['hide'] = array( 'create_tutorial_hidden' );
			}

			if( static::modPermission( 'feature', NULL, $container ) )
			{
				$options['feature'] = 'create_tutorial_featured';
				$toggles['feature'] = array( 'create_tutorial_featured' );
			}

			if( \count( $options ) or \count ( $toggles ) )
			{
				$return['tutorial_state'] = new \IPS\Helpers\Form\CheckboxSet( 'tutorial_create_state', array(), FALSE, array(
					'options' 	=> $options,
					'toggles'	=> $toggles,
					'multiple'	=> TRUE
				) );	
			}
		}

		$dims = TRUE;
		if( $container->allow_images )
		{
			$postKey = ( $item ) ? $item->post_key : md5( uniqid() );
			$dims = $item ? explode( 'x', $container->image_maxdims ) : array( 0,0 );
			$return['image'] = new \IPS\Helpers\Form\Upload( 'a_image', ( ( $item AND $item->image ) ? \IPS\File::get( 'tutorials_Image', $item->image ) : NULL ), FALSE, array( 'image' => array( 'maxWidth' => $dims[0], 'maxHeight' => $dims[1] ), 'maxFileSize' => $container->image_max ? ( $container->image_max / 1024 ) : NULL, 'storageExtension' => 'tutorials_Image', 'multiple' => false ), NULL, NULL, NULL, 'a_image' );
		}

		if( isset( $return['auto_follow'] ) )
		{
			$autofollow = $return['auto_follow'];
			unset( $return['auto_follow'] );
			$return['auto_follow'] = $autofollow;
		}

		return $return;
	}

	public function processForm( $values )
	{
		parent::processForm( $values );

		$this->desc 	= $values['a_desc'];
		$this->content 	= $values['a_content'];

		if( isset( $values['tutorial_create_state'] ) )
		{
			if( \in_array( 'lock', $values['tutorial_create_state'] ) )
			{
				$this->locked = 1;	
			}

			if( \in_array( 'hide', $values['tutorial_create_state'] ) )
			{
				$this->approved = -1;
			}

			if( \in_array( 'pin', $values['tutorial_create_state'] ) )
			{
				$this->pinned = 1;
			}

			if( \in_array( 'feature', $values['tutorial_create_state'] ) )
			{
				$this->featured = 1;
			}
		}

		if( isset( $values['tutorial_author'] ) AND $values['tutorial_author']->member_id )
		{
			$this->mid = $values['tutorial_author']->member_id;
		}

		if( ( $values['a_image'] ) AND $values['a_image'] )
		{
			$this->image = (string) $values['a_image'];
		}
		else
		{
			$this->image = NULL;
			$this->thumb = NULL;
		}

		$this->save();

		if( $this->image )
		{
			try
			{
				$original = \IPS\File::get( 'tutorials_Image', $this->image );
				\IPS\Db::i()->update( 'tutorials_articles', array( 'a_thumb' => (string) $original->thumbnail( 'tutorials_Image' ) ), array( 'a_id=?', $this->id ) );
				$this->save();
			}
			catch( \Exception $e ){}
		}

		$cfields = array();

		foreach( $this->container()->cfields as $field )
		{
			$helper							 = $field->buildHelper();
			
			$cfields[ "field_{$field->id}" ] = $helper::stringValue( $values[ "tutorials_field_{$field->id}" ] );
			
			if( $helper instanceof \IPS\Helpers\Form\Editor )
			{
				$field->claimAttachments( $this->id );
			}
		}

		if( !empty( $cfields ) )
		{
			\IPS\Db::i()->insert( 'tutorials_ccontent', array_merge( array( 'art_id' => $this->id, 'updated' => time() ), $cfields ), TRUE );
		}

		$this->container()->setLastArticle();
		$this->container()->save();

		\IPS\File::claimAttachments( 'tutorials-new-article', $this->id );
	}

	protected function processBeforeCreate( $values )
	{
		parent::processBeforeCreate( $values );
	}

	protected function processAfterCreate( $comment, $values )
	{
		if( $this->image )
		{
			try
			{
				$original = \IPS\File::get( 'tutorials_Image', $this->image );
				$this->thumb = (string) $original->thumbnail( 'tutorials_Image' );
				$this->save();
			}
			catch( \Exception $e ){}
		}

		if( \IPS\tutorials\Store::canUsePoints() AND \IPS\Settings::i()->tutorials_mshop AND $this->approved AND $this->container()->mshop_points > 0 )
		{
			$pointsName = \IPS\Settings::i()->ms_points_name;
			$log = sprintf( \IPS\Member::loggedIn()->language()->get( 'tutorials_points_added' ), $pointsName, $this->url(), $this->name );
			$this->author()->addPoints( $this->container()->mshop_points, $log );
		}

		parent::processAfterCreate( $comment, $values );
	}

	public function logDelete( $member = NULL )
	{
		parent::logDelete( $member );

		if( $topic = $this->topic() and $this->container()->delete_topic )
		{
			$topic->logDelete( $member );
		}
	}
	
	public function delete()
	{
		if( $topic = $this->topic() and $this->container()->delete_topic )
		{
			$topic->delete();
		}

		parent::delete();

		\IPS\Db::i()->delete( 'tutorials_ccontent', array( 'art_id=?', $this->id ) );
		\IPS\Db::i()->delete( 'core_ratings', array( 'item_id=?', $this->id ) );

		$this->container()->setLastArticle();
		$this->container()->save();
	}

	public static function canTag( \IPS\Member $member = NULL, \IPS\Node\Model $container = NULL )
	{
		return parent::canTag( $member, $container ) and ( $container === NULL or !$container->tags_disabled );
	}
	
	public static function canPrefix( \IPS\Member $member = NULL, \IPS\Node\Model $container = NULL )
	{
		return parent::canPrefix( $member, $container ) and ( $container === NULL or !$container->tags_noprefixes );
	}
	
	public static function definedTags( \IPS\Node\Model $container = NULL )
	{
		if( $container and $container->predefined_tags )
		{
			return explode( ',', $container->predefined_tags );
		}
		
		return parent::definedTags( $container );
	}

	protected $firstTimeApproval = FALSE;
	public function unhide( $member=NULL )
	{
		if( $this->hidden() === 1 )
		{
			$this->firstTimeApproval = TRUE;
		}

		if( $this->firstTimeApproval )
		{
			if( \IPS\tutorials\Store::canUsePoints() AND \IPS\Settings::i()->tutorials_mshop AND $this->hidden() === 1 AND $this->container()->mshop_points > 0 )
			{
				$pointsName = \IPS\Settings::i()->ms_points_name;
				$log = sprintf( \IPS\Member::loggedIn()->language()->get( 'tutorials_points_added' ), $pointsName, $this->url(), $this->name );
				$this->author()->addPoints( $this->container()->mshop_points, $log );
			}
		}

		parent::unhide( $member );
	}

	protected function createNotification( $extra=NULL )
	{
		return new \IPS\Notification( \IPS\Application::load( 'core' ), 'new_content', $this, array( $this ) );
	}

	public function processAfterEdit( $values )
	{
		parent::processAfterEdit( $values );

		$this->updated = time();
		$this->save();
	}

	public function canChangeAuthor( \IPS\Member $member = NULL )
	{
		return static::modPermission( 'edit', $member, $this->container() );
	}

	public function canChangeTutorialDate( \IPS\Member $member = NULL )
	{
		return static::modPermission( 'can_change_tutorial_date', $member, $this->container() );
	}

	public function canViewReaders( \IPS\Member $member = NULL )
	{
		if( !$this->container()->log )
		{
			return FALSE;
		}

		$member = $member ?: \IPS\Member::loggedIn();

		if( $member == $this->author() and $this->container()->log_author )
		{
			return TRUE;
		}

		return $member->group['g_tutorials_viewreaders'];
	}

	public function displayShareLinks()
	{
		return $this->container()->sharing;
	}

	public function canRate( \IPS\Member $member = NULL )
	{
		return $this->container()->rating and parent::canRate( $member );
	}

	public function topic( $checkPerms=TRUE )
	{
		if( \IPS\Application::appIsEnabled('forums') AND $this->topic AND ( $this->container()->forum OR \IPS\Settings::i()->tutorials_topics_forum ) )
		{
			try
			{
				return $checkPerms ? \IPS\forums\Topic::loadAndCheckPerms( $this->topic ) : \IPS\forums\Topic::load( $this->topic );
			}
			catch ( \OutOfRangeException $e )
			{
				return NULL;
			}
		}
		
		return NULL;
	}

	public function hasAttachments()
	{
		try
		{
			if( \IPS\Db::i()->select( '*', 'core_attachments_map', array( 'id1=? AND location_key=?', $this->id, 'tutorials_Article' ) )->join( 'core_attachments', 'core_attachments_map.attachment_id=core_attachments.attach_id' )->first() > 0 )
			{
				return TRUE;
			}
			else
			{
				return FALSE;
			}
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	public function getVideos()
	{
		try
		{
			$videos = iterator_to_array( \IPS\Db::i()->select( '*', 'tutorials_articles_videos', array( 'a_video_tutorial_id=?', $this->id ), 'a_video_id ASC' )->setKeyField( 'a_video_id' ) );

			return $videos;
		}
		catch( \UnderflowException $e )
		{
			return array();
		}
	}

	public function getVideo( $id )
	{
		try
		{
			$video = \IPS\Db::i()->select( '*', 'tutorials_articles_videos', array( 'a_video_id=?', $id ) )->first();

			return $video;
		}
		catch( \UnderflowException $e )
		{
			return array();
		}
	}

	public function getVideoIdForImage( $url )
	{
		$videoHost 	= parse_url( $url );
		$image		= '';

		switch( $videoHost['host'] )
		{
			case 'vimeo.com':
				$imgid  = (int) mb_substr( parse_url( $url, PHP_URL_PATH ), 1 );
				$data   = \unserialize( file_get_contents( 'http://vimeo.com/api/v2/video/' . $imgid . '.php' ) );
				$img  	= $data[0]['thumbnail_medium'];
				$image	= '<img src="' . $img . '">';
			break;
			case 'www.youtube.com';
				$queryString = parse_url( $url, PHP_URL_QUERY );
				parse_str( $queryString, $params );
				if( isset( $params['v'] ) )
				{
					$image = '<img src="https://img.youtube.com/vi/'. $params['v'] . '/hqdefault.jpg">';
				}
			break;
		}

		return $image;
	}

	public function getVideoId( $url )
	{
		$videoHost = parse_url( $url );

		switch( $videoHost['host'] )
		{
			case 'vimeo.com':
				$id = (int) mb_substr(parse_url( $url, PHP_URL_PATH ), 1 );
			break;
			case 'www.youtube.com';
				$queryString = parse_url( $url, PHP_URL_QUERY );
				parse_str( $queryString, $params );
				if( isset( $params['v'] ) )
				{
					$id = $params['v']; 
				}
			break;
		}

		return $id;
	}

	public static function reactionType()
	{
		return 'a_id';
	}

	public static function supportedMetaDataTypes()
	{
		return array( 'core_FeaturedComments', 'core_ContentMessages', 'core_ItemModeration' );
	}

	public function embedImage()
	{
		return $this->thumb  ? \IPS\File::get( 'tutorials_Image', $this->thumb ) : NULL;
	}

	public function embedContent( $params )
	{
		\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'tutorials.css', 'tutorials', 'front' ) );
		return \IPS\Theme::i()->getTemplate( 'global', 'tutorials', 'front' )->embedArticle( $this, $this->url()->setQueryString( $params ), $this->embedImage() );
	}

	public function contentImages( $limit = NULL, $ignorePermissions = false )
	{
		$attachments = parent::contentImages( $limit ) ?: array();

		if( $this->thumb )
		{
			$attachments[] = array( 'tutorials_Image' => $this->thumb );
		}

		$idColumn = static::$databaseColumnId;
		$internal = NULL;

		if( isset( static::$databaseColumnMap['content'] ) )
		{
			$internal = \IPS\Db::i()->select( 'attachment_id', 'core_attachments_map', array( 'location_key=? and id1=?', 'tutorials_Article', $this->$idColumn ) );
		}

		if( $internal )
		{
			foreach( \IPS\Db::i()->select( '*', 'core_attachments', array( array( 'attach_id IN(?)', $internal ), array( 'attach_is_image=1' ) ), 'attach_id ASC', $limit ) as $row )
			{
				$attachments[] = array( 'core_Attachment' => $row['attach_location'] );
			}
		}

		return \count( $attachments ) ? $attachments : NULL;
	}


	public static function topicIdColumn()
	{
		return 'topic';
	}

	public static function containerForumIdColumn()
	{
		return 'forum';
	}

	function getTopicTitle()
	{
		return $this->container()->prefix . $this->name . $this->container()->suffix;
	}

	function getTopicContent()
	{
		return \IPS\Theme::i()->getTemplate( 'submit', 'tutorials', 'front' )->topic( $this );
	}

	public static function searchResultSnippet( array $indexData, array $authorData, array $itemData, array $containerData = NULL, array $reputationData, $reviewRating, $view )
	{
		$screenshot = isset( $itemData['a_thumb'] ) ? $itemData['a_thumb'] : NULL;

		if( !$screenshot AND $view == 'condensed' )
		{
			return parent::searchResultSnippet( $indexData, $authorData, $itemData, $containerData, $reputationData, $reviewRating, $view );
		}

		$url = \IPS\Http\Url::internal( static::$urlBase . $indexData['index_item_id'], 'front', static::$urlTemplate, \IPS\Http\Url\Friendly::seoTitle( $indexData['index_title'] ?: $itemData[ static::$databasePrefix . static::$databaseColumnMap['title'] ] ) );

		return \IPS\Theme::i()->getTemplate( 'global', 'tutorials', 'front' )->searchResultArticleSnippet( $indexData, $itemData, $screenshot, $url, $view == 'condensed' );
	}

	public static function getWidgetSortOptions()
	{
		$sortOptions = parent::getWidgetSortOptions();

		$sortOptions['_rand'] = 'sort_rand';

		return $sortOptions;
	}

	public function lastModificationDate()
	{
		if( !$this->container()->allow_comments AND $this->container()->allow_reviews )
		{
			return \IPS\DateTime::ts( $this->mapped( 'last_review' ) );
		}

		return parent::lastModificationDate();
	}

	public function canConvertReview( \IPS\Member $member = NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();

		if
		(
			$member->modPermission( 'can_convert_tutorial_rev_com' )
			and
			(
				( $member->modPermission( \IPS\tutorials\Category::$modPerm ) === TRUE or $member->modPermission( \IPS\tutorials\Category::$modPerm ) === -1 )
				or
				(
					\is_array( $member->modPermission( \IPS\tutorials\Category::$modPerm ) )
					and
					\in_array( $this->container()->_id, $member->modPermission( \IPS\tutorials\Category::$modPerm ) )
				)
			)
			and $this->container()->allow_comments
		)
		{
			return TRUE;
		}

		return FALSE;
	}

	/**
	 * Get output for API
	 *
	 * @return	array
	 * @apiresponse	int								id					ID number
	 * @apiresponse	string							title				Title
	 * @apiresponse	\IPS\links\Category				category			Category
	 * @apiresponse	\IPS\Member						author				Author
	 * @apiresponse	datetime						date				When the link was created
	 * @apiresponse	string							description			Description
	 * @apiresponse	string							version				Current version number
	 * @apiresponse	string							changelog			Description of what changed between this version and the previous one
	 * @apiresponse	int								comments			Number of comments
	 * @apiresponse	int								reviews				Number of reviews
	 * @apiresponse	int								views				Number of views
	 * @apiresponse	string							prefix				The prefix tag, if there is one
	 * @apiresponse	[string]						tags				The tags
	 * @apiresponse	bool							locked				Link is locked
	 * @apiresponse	bool							hidden				Link is hidden
	 * @apiresponse	bool							pinned				Link is pinned
	 * @apiresponse	bool							featured			Link is featured
	 * @apiresponse	string							url					URL
	 */
	public function apiOutput()
	{
		return array(
			'id'				=> $this->id,
			'title'				=> $this->name,
			'category'			=> $this->container()->apiOutput(),
			'author'			=> $this->author()->apiOutput(),
			'date'				=> \IPS\DateTime::ts( $this->added )->rfc3339(),
			'desc'				=> $this->desc,
			'description'		=> $this->content(),
			'comments'			=> $this->comments,
			'reviews'			=> $this->reviews,
			'views'				=> $this->views,
			'prefix'			=> $this->prefix(),
			'tags'				=> $this->tags(),
			'locked'			=> (bool) $this->locked(),
			'hidden'			=> (bool) $this->hidden(),
			'pinned'			=> (bool) $this->mapped('pinned'),
			'featured'			=> (bool) $this->mapped('featured'),
			'url'				=> (string) $this->url(),
		);
	}
}