<?php

namespace IPS\tutorials;

/* To prevent PHP errors (extending class does not exist) revealing path */
if( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

class _Category extends \IPS\Node\Model implements \IPS\Node\Permissions
{
 	use \IPS\Content\ClubContainer;
 	use \IPS\Node\Colorize;
    use \IPS\Node\Statistics;
	protected static $multitons;
	public static $databaseTable = 'tutorials_cats';
	public static $databasePrefix = 'c_';
	public static $databaseColumnOrder = 'order';
	public static $databaseColumnParent = 'parent';
	public static $nodeTitle = 'categories';

	protected static $restrictions = array(
		'app'		=> 'tutorials',
		'module'	=> 'tutorials',
		'prefix'	=> 'categories_'
	);
	
	public static $permApp = 'tutorials';
	public static $permType = 'category';
	public static $permissionMap = array(
		'view' 				=> 'view',
		'read'				=> 2,
		'add'				=> 3,
		'reply'				=> 4,
		'review'			=> 6,
		'post'				=> 5
	);

	public static $titleLangPrefix = 'tutorials_category_';
	public static $descriptionLangSuffix = '_desc';
	public static $modPerm = 'tutorials_cats';
	public static $contentItemClass = 'IPS\tutorials\Article';
	public static $permissionLangPrefix = 'perm_tut_';
	public static $databaseColumnEnabledDisabled = 'open';
	protected $_url	= NULL;
	public static $urlBase = 'app=tutorials&module=tutorials&controller=browse&id=';
	public static $urlTemplate = 'tutorials_cat';
	public static $seoTitleColumn = 'name_seo';

	public function get_name_seo()
	{
		if( !$this->_data['name_seo'] )
		{
			$this->name_seo	= \IPS\Http\Url::seoTitle( \IPS\Lang::load( \IPS\Lang::defaultLanguage() )->get( 'tutorials_category_' . $this->id ) );
			$this->save();
		}

		return $this->_data['name_seo'] ?: \IPS\Http\Url::seoTitle( \IPS\Lang::defaultLanguage() )->get( 'tutorials_category_' . $this->id );
	}

	protected function get__enabled()
	{
		return $this->open;
	}

	protected function set__enabled( $enabled )
	{
		$this->open	= $enabled;
	}

	public function get__sortBy()
	{
		return $this->artorder;
	}

	public function get__sortOrder()
	{
		return $this->artsort;
	}

	public function form( &$form )
	{
		$customFields = array();
		foreach( \IPS\Db::i()->select( 'cf_id', 'tutorials_cfields', NULL, 'cf_position' ) as $fieldId )
		{
			$customFields[ $fieldId ] = \IPS\Member::loggedIn()->language()->addToStack( "tutorials_field_{$fieldId}" );
		}

		$form->addTab( 'settings' );
		$form->addHeader( 'settings' );
		$form->add( new \IPS\Helpers\Form\Translatable( 'c_name', NULL, TRUE, array( 'app' => 'tutorials', 'key' => ( $this->id ? "tutorials_category_{$this->id}" : NULL ) ) ) );
		$form->add( new \IPS\Helpers\Form\Translatable( 'c_desc', NULL, FALSE, array(
			'app'		=> 'tutorials',
			'key'		=> ( $this->id ? "tutorials_category_{$this->id}_desc" : NULL ),
			'editor'	=> array(
				'app'			=> 'tutorials',
				'key'			=> 'Categories',
				'autoSaveKey'	=> ( $this->id ? "tutorials-cat-{$this->id}" : "tutorials-new-cat" ),
				'attachIds'		=> $this->id ? array( $this->id, NULL, 'description' ) : NULL, 'minimize' => 'c_desc_placeholder'
			)
		) ) );

		$form->add( new \IPS\Helpers\Form\Node( 'c_parent', $this->id ? $this->parent : ( \IPS\Request::i()->c_parent ?: 0 ), FALSE, array(
			'class'		      => '\IPS\tutorials\Category',
			'disabled'	      => false,
			'zeroVal'         => 'node_no_parent',
			'permissionCheck' => function( $node )
			{
				if( ! isset( \IPS\Request::i()->id ) )
				{
					return true;
				}
				
				return $node->id != \IPS\Request::i()->id;
			}
		) ) );

		$form->add( new \IPS\Helpers\Form\YesNo( 'c_use_feature_color', $this->feature_color ? 1 : 0, FALSE, array( 'togglesOn' => array( 'c_feature_color' ) ), NULL, NULL, NULL, 'c_use_feature_color' ) );
		$form->add( new \IPS\Helpers\Form\Color( 'c_feature_color', $this->feature_color ?: '', FALSE, array(), NULL, NULL, NULL, 'c_feature_color' ) );

		if( !empty( $customFields ) )
		{
			$form->add( new \IPS\Helpers\Form\CheckboxSet( 'cfields', explode( ',', $this->_data['cfields'] ), FALSE, array( 'options' => $customFields, 'multiple' => TRUE ), NULL, NULL, NULL, 'cfields' ) );
		}

		$form->add( new \IPS\Helpers\Form\Upload( 'c_icon', $this->icon ? \IPS\File::get( 'tutorials_Icon', $this->icon ) : NULL, FALSE, array( 'image' => TRUE, 'storageExtension' => 'tutorials_Icon' ), NULL, NULL, NULL, 'c_icon' ) );

		$form->add( new \IPS\Helpers\Form\Upload( 'c_image', $this->image ? \IPS\File::get( 'tutorials_catImage', $this->image ) : NULL, FALSE, array( 'image' => TRUE, 'storageExtension' => 'tutorials_catImage' ), NULL, NULL, NULL, 'c_image' ) );

		$form->addHeader( 'tutorials_category_display' );
		$form->add( new \IPS\Helpers\Form\Select( 'c_artorder', ( $this->artorder ) ? $this->artorder : 'updated', FALSE, array( 'options' => array( 'title' => 'c_ordering_title', 'views' => 'c_ordering_views', 'date' => 'c_ordering_date', 'updated' => 'c_ordering_update', 'comments' => 'c_ordering_comments' ) ), NULL, NULL, NULL, 'c_artorder' ) );
		$form->add( new \IPS\Helpers\Form\Select( 'c_artsort', ( $this->artsort ) ? $this->artsort : 'DESC', FALSE, array( 'options' => array( 'DESC' => 'c_sorting_descending', 'ASC' => 'c_sorting_ascending' ) ), NULL, NULL, NULL, 'c_artsort' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_sharing', $this->sharing ? $this->sharing : TRUE, FALSE, array(), NULL, NULL, NULL, 'c_sharing' ) );

		$form->addHeader( 'category_logs' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_log', $this->id ? $this->log : 0, FALSE, array( 'togglesOn' => array(
				//'c_log',
				'c_log_days',
				'c_log_author'
		) ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'c_log_days', $this->log === NULL ? -1 : $this->log_days, FALSE, array( 'min' => 1, 'unlimited' => -1 ), NULL, NULL, NULL, 'c_log_days' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_log_author', $this->log_author ? $this->log_author : 0, FALSE, array(), NULL, NULL, NULL, 'c_log_author' ) );

		$form->addTab( 'category_submissions' );
		$form->addHeader( 'tutorials' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_article_moderation', $this->id ? $this->article_moderation : FALSE, FALSE, array(), NULL, NULL, NULL, 'c_article_moderation' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_desc_required', $this->id ? $this->desc_required : FALSE, FALSE, array(), NULL, NULL, NULL, 'c_desc_required' ) );

		$form->addHeader( 'c_image_title' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_allow_images', $this->id ? $this->allow_images : TRUE, FALSE, array( 'togglesOn' => array( 'c_image_max', 'c_image_maxdims' ) ), NULL, NULL, NULL, 'c_allow_images' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'c_image_max', $this->image_max ? $this->image_max : '512', FALSE, array( 'unlimited' => 0 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('filesize_raw_k'), 'c_image_max' ) );

		$size = $this->id ? explode( 'x', $this->image_maxdims ) : array( 100, 100 );
		$form->add( new \IPS\Helpers\Form\WidthHeight( 'c_image_maxdims', $size, FALSE, array( 'unlimited' => array( 0, 0 ) ), NULL, NULL, NULL, 'c_image_maxdims' ) );

		$form->addHeader( 'tutorials_videos' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_allow_videos', $this->id ? $this->allow_videos : TRUE ) );

		$form->addHeader( 'tutorial_comments' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_allow_comments', $this->id ? $this->allow_comments : TRUE, FALSE, array( 'togglesOn' => array( 'c_comments_moderation' ) ), NULL, NULL, NULL, 'c_allow_comments' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_comments_moderation',  $this->id ? $this->comments_moderation : FALSE, FALSE, array(), NULL, NULL, NULL, 'c_comments_moderation' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_allow_reviews', $this->id ? $this->allow_reviews : FALSE, FALSE, array( 'togglesOn' => array( 'c_moderate_reviews' ) ), NULL, NULL, NULL, 'c_allow_reviews' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_moderate_reviews', $this->id ? $this->moderate_reviews : FALSE, FALSE, array(), NULL, NULL, NULL, 'c_moderate_reviews' ) );

		if( \IPS\tutorials\Store::canUsePoints() AND \IPS\Settings::i()->tutorials_mshop )
		{
			$form->addHeader( 'tutorials_tab_membersshop_enable' );
			$form->add( new \IPS\Helpers\Form\Number( 'c_mshop_points', $this->id ? $this->mshop_points : 0, FALSE, array( 'unlimited' => 0, 'unlimitedLang' => 'c_mshop_points_donot_earn' ) ) );
		}

		if( \IPS\Settings::i()->tags_enabled )
		{
			$form->addHeader( 'category_tags' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'c_tags_disabled', !$this->tags_disabled, FALSE, array( 'togglesOn' => array( 'c_tags_noprefixes', 'c_predefined_tags' ) ), NULL, NULL, NULL, 'c_tags_disabled' ) );
			$form->add( new \IPS\Helpers\Form\YesNo( 'c_tags_noprefixes', !$this->tags_noprefixes, FALSE, array(), NULL, NULL, NULL, 'c_tags_noprefixes' ) );

			if( !\IPS\Settings::i()->tags_open_system )
			{
				$form->add( new \IPS\Helpers\Form\Text( 'c_predefined_tags', $this->predefined_tags, FALSE, array( 'autocomplete' => array( 'unique' => 'true' ), 'nullLang' => 'c_predefined_tags_unlimited' ), NULL, NULL, NULL, 'c_predefined_tags' ) );
			}
		}

		$form->addTab( 'category_templates' );

		$form->add( new \IPS\Helpers\Form\Text( 'c_def_title', $this->def_title, FALSE, array(), NULL, NULL, NULL, 'c_def_title' ) );
		$form->add( new \IPS\Helpers\Form\TextArea( 'c_def_desc', $this->id ? $this->def_desc : '', FALSE, array( 'rows' => 3, 'maxLength' => \IPS\Settings::i()->tutorials_maxpreview ) ) );
		$form->add( new \IPS\Helpers\Form\Editor( 'c_def_content', $this->id ? $this->def_content : '', FALSE, array(
				'app'			=> 'tutorials',
				'key'			=> 'Categories',
				'autoSaveKey'	=> ( $this->id ? "tutorials-cat-{$this->id}-def_content" : "tutorials-def_content" ),
				'attachIds'		=> $this->id ? array( $this->id, NULL, 'defcontent' ) : NULL, 'minimize' => 'c_def_content_placeholder'
		) ) );

		if( \IPS\Application::appIsEnabled( 'forums' ) )
		{
			$form->addTab( 'category_forums_integration' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'c_forum_tut_on', $this->forum_tut_on, FALSE, array( 'togglesOn' => array(
				'c_forum',
				'c_prefix',
				'c_suffix',
				'c_delete_topic'
			) ) ) );
			$form->add( new \IPS\Helpers\Form\Node( 'c_forum', $this->forum, FALSE, array( 'class' => 'IPS\forums\Forum', 'clubs' => TRUE ), NULL, NULL, NULL, 'c_forum' ) );
			$form->add( new \IPS\Helpers\Form\Text( 'c_prefix', $this->prefix, FALSE, array(), NULL, NULL, NULL, 'c_prefix' ) );
			$form->add( new \IPS\Helpers\Form\Text( 'c_suffix', $this->suffix, FALSE, array(), NULL, NULL, NULL, 'c_suffix' ) );
			$form->add( new \IPS\Helpers\Form\YesNo( 'c_delete_topic', $this->id ? $this->delete_topic : 1, FALSE, array(), NULL, NULL, NULL, 'c_delete_topic' ) );
		}

		if( \IPS\Application::appIsEnabled( 'forums' ) )
		{
			$form->addTab( 'category_forums_forum' );
			$form->addMessage( \IPS\Member::loggedIn()->language()->addToStack("c_forum_support_text") );
			$form->add( new \IPS\Helpers\Form\YesNo( 'c_forum_support_on', $this->forum_support_on, FALSE, array( 'togglesOn' => array(	'c_forum_support' ) ) ) );
			$form->add( new \IPS\Helpers\Form\Node( 'c_forum_support', $this->forum_support, FALSE, array( 'class' => 'IPS\forums\Forum', 'clubs' => TRUE ), NULL, NULL, NULL, 'c_forum_support' ) );
		}
	}
	
	public function saveForm( $values )
	{
		if( !$this->id )
		{
			$this->save();
			\IPS\File::claimAttachments( 'tutorials-new-cat', $this->id, NULL, 'description', TRUE );
			\IPS\File::claimAttachments( 'tutorials-new-cat-defcontent', $this->id, NULL, 'def_content', TRUE );
		}
		else
		{
			\IPS\File::claimAttachments( 'tutorials-cat-' . $this->id, $this->id, NULL, 'description', TRUE );
			\IPS\File::claimAttachments( 'tutorials-cat-defcontent' . $this->id, $this->id, NULL, 'def_content', TRUE );
		}

		foreach( array( 'c_name' => "tutorials_category_{$this->id}", 'c_desc' => "tutorials_category_{$this->id}_desc" ) as $fieldKey => $langKey )
		{
			if( isset( $values[ $fieldKey ] ) )
			{
				\IPS\Lang::saveCustom( 'tutorials', $langKey, $values[ $fieldKey ] );
				
				if( $fieldKey === 'c_name' )
				{
					$this->name_seo = \IPS\Http\Url::seoTitle( $values[ $fieldKey ][ \IPS\Lang::defaultLanguage() ] );
				}
				
				unset( $values[ $fieldKey ] );
			}
		}

		$values['c_image_maxdims'] = implode( 'x', $values[ 'c_image_maxdims' ] );

		foreach( array( 'c_tags_disabled', 'c_tags_noprefixes' ) as $k )
		{
			if( isset( $values[ $k ] ) )
			{
				$values[ $k ] = !$values[ $k ];
			}
		}

		if( isset( $values['c_parent'] ) )
		{
			$values['c_parent'] = $values['c_parent'] ? \intval( $values['c_parent']->id ) : 0;
		}

		foreach( array( 'c_def_title', 'c_def_desc', 'c_def_content', 'c_prefix', 'c_suffix' ) as $k )
		{
			if( isset( $values[ $k ] ) )
			{
				$values[ $k ] = $values[ $k ] ?: NULL;
			}
		}

		if( isset( $values['c_use_feature_color'] ) )
		{
			if( ! $values['c_use_feature_color'] )
			{
				$values['c_feature_color'] = NULL;
			}

			unset( $values['c_use_feature_color'] );
		}

		$values['c_artorder'] = $values['c_artorder'] != 'update' ? $values['c_artorder'] : 'update';
		$values['c_artsort']  = $values['c_artsort']  != 'DESC'   ? 'ASC' : 'DESC';

		if( $values['c_forum_tut_on'] and isset( $values['c_forum'] ) )
		{
			$values['c_forum'] = ( $values['c_forum'] instanceof \IPS\Node\Model ) ? \intval( $values['c_forum']->id ) : \intval( $values['c_forum'] );
		}
		else
		{
			$values['c_forum'] = 0;
		}

		if( $values['c_forum_support_on'] and isset( $values['c_forum_support'] ) )
		{
			$values['c_forum_support'] = ( $values['c_forum_support'] instanceof \IPS\Node\Model ) ? \intval( $values['c_forum_support']->id ) : \intval( $values['c_forum_support'] );
		}
		else
		{
			$values['c_forum_support'] = 0;
		}

		parent::saveForm( $values );
	}

	protected $_customFields = NULL;
	protected function get_cfields()
	{
		if( $this->_customFields === NULL )
		{
			$this->_customFields = array();
			if( $this->_data['cfields'] )
			{
				foreach( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( '*', 'tutorials_cfields', array( \IPS\Db::i()->in( 'cf_id', explode( ',', $this->_data['cfields'] ) ) ), 'cf_position ASC' ), 'IPS\tutorials\Field' ) AS $field )
				{
					$this->_customFields[ $field->id ] = $field;
				}
			}
		}

		return $this->_customFields;
	}

	public function url()
	{
		if( $this->_url === NULL )
		{
			if( !\IPS\Settings::i()->tutorials_filterbyletters )
			{
				$this->_url = \IPS\Http\Url::internal( "app=tutorials&module=tutorials&controller=browse&id={$this->_id}", 'front', 'tutorials_cat', $this->name_seo );
			}
			else
			{
				if( !isset( \IPS\Request::i()->letter ) )
				{
					$this->_url = \IPS\Http\Url::internal( "app=tutorials&module=tutorials&controller=browse&id={$this->_id}", 'front', 'tutorials_cat', $this->name_seo );
				}
				else
				{
					$letter = \IPS\Request::i()->letter;
					$this->_url = \IPS\Http\Url::internal( "app=tutorials&module=tutorials&controller=browse&id={$this->_id}&letter={$letter}", 'front', 'tutorials_cat', $this->name_seo );
				}
			}
		}

		return $this->_url;
	}

	public function getLastCommentTime( \IPS\Member $member = NULL )
	{
		return $this->last_time ? \IPS\DateTime::ts( $this->last_time ) : NULL;
	}

	public function setLastArticle( \IPS\tutorials\Article $article=NULL )
	{
		if( $article === NULL )
		{
			try
			{
				$article = \IPS\tutorials\Article::constructFromData( \IPS\Db::i()->select( '*', 'tutorials_articles', array( 'a_cat=? AND a_approved=1', $this->id ), 'a_date DESC', 1 )->first() );
			}
			catch ( \UnderflowException $e )
			{
				$this->last_id		= 0;
				$this->last_time	= 0;
				return;
			}
		}
	
		$this->last_id		= $article->id;
		$this->last_time 	= $article->date;
	}

	public function lastArticle()
	{
		try
		{
			$latestArticle = \IPS\tutorials\Article::load( $this->last_id );
		}
		catch ( \OutOfRangeException $e )
		{
			$latestArticle = NULL;
		}

		foreach( $this->children() as $child )
		{
			$childLatest = $child->lastArticle();

			if( $childLatest !== NULL AND ( $latestArticle === NULL OR $childLatest->updated > $latestArticle->updated ) )
			{
				$latestArticle = $childLatest;
			}
		}

		return $latestArticle;
	}

	public function setLastComment( \IPS\Content\Comment $comment=NULL )
	{
		$this->setLastArticle();
	}

	public function delete()
	{
		\IPS\File::unclaimAttachments( 'tutorials_Categories', $this->id );
		parent::delete();
		
		foreach( array( 'c_name' => "tutorials_category_{$this->id}", 'c_desc' => "tutorials_category_{$this->id}_description", 'c_def_content' => "tutorials_category_{$this->id}_def_content" ) as $fieldKey => $langKey )
		{
			\IPS\Lang::deleteCustom( 'tutorials', $langKey );
		}
	}

	public static function nodeTableTemplate()
	{
		return array( \IPS\Theme::i()->getTemplate( 'browse', 'tutorials' ), 'categoryRow' );
	}

	public static function canOnAny( $permission, $member=NULL, $where=array(), $considerPostBeforeRegistering = TRUE )
	{
		$member	= ( $member === NULL ) ? \IPS\Member::loggedIn() : $member;
		$data 	= \IPS\tutorials\Members::getMemberData( $member->member_id );

		if( $member->member_id AND $data['block_submissions'] == 0 )
		{
			return FALSE;
		}

		return parent::canOnAny( $permission, $member, $where, $considerPostBeforeRegistering );
	}

	public static function getMemberView()
	{
		$method = ( isset( \IPS\Request::i()->cookie['tutorials_view'] ) ) ? \IPS\Request::i()->cookie['tutorials_view'] : NULL;

		if( !$method )
		{
			try
			{
				$method = \IPS\Db::i()->select( 'method', 'tutorials_view_method', array( 'member_id=?', \IPS\Member::loggedIn()->member_id ) )->first();
				\IPS\Request::i()->setCookie( 'tutorials_view', $method, ( new \IPS\DateTime )->add( new \DateInterval( 'P1Y' ) ) );
			}
			catch( \UnderFlowException $e ){}
		}

		if( !$method )
		{
			$method = \IPS\Settings::i()->tutorials_index_type;
		}
		
		return $method;
	}

	public static function clubAcpTitle()
	{
		return 'tutorials_categories';
	}

	public function clubForm( \IPS\Helpers\Form $form )
	{
		$itemClass = static::$contentItemClass;
		$form->addTab('editor_settings_general');
		$form->add( new \IPS\Helpers\Form\Text( 'club_node_name', $this->_id ? $this->_title : \IPS\Member::loggedIn()->language()->addToStack( $itemClass::$title . '_pl' ), TRUE, array( 'maxLength' => 255 ) ) );
		$form->add( new \IPS\Helpers\Form\Editor( 'club_node_description', $this->_id ? \IPS\Member::loggedIn()->language()->addToStack( static::$titleLangPrefix . $this->_id . '_desc' ) : NULL, FALSE, array( 'app' => 'tutorials', 'key' => 'Admin', 'autoSaveKey' => ( $this->id ? "tutorials-cat-{$this->id}" : "tutorials-new-cat" ), 'attachIds' => $this->id ? array( $this->id, NULL, 'description' ) : NULL, 'minimize' => 'c_desc' ) ) );
		$form->addTab('category_settings');
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_desc_required', $this->id ? $this->desc_required : FALSE, FALSE, array(), NULL, NULL, NULL, 'c_desc_required' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_allow_comments', $this->id ? $this->allow_comments : TRUE, FALSE, array(), NULL, NULL, NULL, 'c_allow_comments' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'c_allow_reviews', $this->id ? $this->allow_reviews : TRUE, FALSE, array(), NULL, NULL, NULL, 'c_allow_reviews' ) );

		if( \IPS\Application::appIsEnabled( 'forums' ) )
		{
			$form->addTab( 'category_forums_integration' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'c_forum_tut_on', $this->forum_tut_on, FALSE, array( 'togglesOn' => array(
				'c_forum',
				'c_prefix',
				'c_suffix',
				'c_delete_topic'
			) ) ) );
			$form->add( new \IPS\Helpers\Form\Node( 'c_forum', $this->forum, FALSE, array( 'class' => 'IPS\forums\Forum', 'clubs' => TRUE ), NULL, NULL, NULL, 'c_forum' ) );
			$form->add( new \IPS\Helpers\Form\Text( 'c_prefix', $this->prefix, FALSE, array(), NULL, NULL, NULL, 'c_prefix' ) );
			$form->add( new \IPS\Helpers\Form\Text( 'c_suffix', $this->suffix, FALSE, array(), NULL, NULL, NULL, 'c_suffix' ) );
			$form->add( new \IPS\Helpers\Form\YesNo( 'c_delete_topic', $this->id ? $this->delete_topic : 1, FALSE, array(), NULL, NULL, NULL, 'c_delete_topic' ) );
		}

		if( \IPS\Application::appIsEnabled( 'forums' ) )
		{
			$form->addTab( 'category_forums_forum' );
			$form->addMessage( \IPS\Member::loggedIn()->language()->addToStack("c_forum_support_text") );
			$form->add( new \IPS\Helpers\Form\YesNo( 'c_forum_support_on', $this->forum_support_on, FALSE, array( 'togglesOn' => array(	'c_forum_support' ) ) ) );
			$form->add( new \IPS\Helpers\Form\Node( 'c_forum_support', $this->forum_support, FALSE, array( 'class' => 'IPS\forums\Forum', 'clubs' => TRUE ), NULL, NULL, NULL, 'c_forum_support' ) );
		}
	}

	public function _saveClubForm( \IPS\Member\Club $club, $values )
	{
		$this->allow_comments 	= $values['c_allow_comments'];
		$this->allow_reviews 	= $values['c_allow_reviews'];
		$this->desc_required	= $values['c_desc_required'];
		$this->artorder			= 'updated';
		$this->artsort			= 'DESC';

		if( $values['c_forum_tut_on'] and isset( $values['c_forum'] ) )
		{
			$this->forum_tut_on = 1;
			$this->forum 		= ( $values['c_forum'] instanceof \IPS\Node\Model ) ? \intval( $values['c_forum']->id ) : \intval( $values['c_forum'] );
		}
		else
		{
			$this->forum_tut_on = 0;
			$this->forum 		= 0;
		}

		if( $values['c_forum_support_on'] and isset( $values['c_forum_support'] ) )
		{
			$this->forum_support_on = 1;
			$this->forum_support 	= ( $values['c_forum_support'] instanceof \IPS\Node\Model ) ? \intval( $values['c_forum_support']->id ) : \intval( $values['c_forum_support'] );
		}
		else
		{
			$this->forum_support_on = 0;
			$this->forum_support 	= 0;
		}
	}

	public static function tutorialsInClubNodes()
	{
		return \IPS\tutorials\Article::getItemsWithPermission( array( array( static::$databasePrefix . static::clubIdColumn() . ' IS NOT NULL' ) ), NULL, 1, 'read', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, TRUE, FALSE, FALSE, TRUE );
	}
}