<?php

namespace IPS\tutorials\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Log Cleanup Task
 */
class _logCleanup extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{	
		foreach( \IPS\Db::i()->select( '*', 'tutorials_cats', 'c_log>0' ) as $cat )
		{
			\IPS\Db::i()->delete( 'tutorials_articles_views', array( 'date<? AND tutorial_id IN(?)', \IPS\DateTime::create()->sub( new \DateInterval( 'P' . $cat['c_log'] . 'D' ) )->getTimestamp(), \IPS\Db::i()->select( 'a_id', 'tutorials_articles', array( 'a_cat=?', $cat['c_id'] ) ) ) );
		}
				
		return NULL;
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}