<?php
/**
 * @brief		featuredTutorials Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	tutorials
 * @since		14 Jan 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\tutorials\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * featuredTutorials Widget
 */
class _featuredTutorials extends \IPS\Widget\PermissionCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'featuredTutorials';
	
	/**
	 * @brief	App
	 */
	public $app = 'tutorials';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
		// Use this to perform any set up and to assign a template that is not in the following format:
		// $this->template( array( \IPS\Theme::i()->getTemplate( 'widgets', $this->app, 'front' ), $this->key ) );
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
		$form = parent::configuration( $form );

 		$form->add( new \IPS\Helpers\Form\Number( 'tuts_to_show_featured', isset( $this->configuration['tuts_to_show_featured'] ) ? $this->configuration['tuts_to_show_featured'] : 5, TRUE ) );
		$form->add( new \IPS\Helpers\Form\Select( 'tuts_exclude_groups_featured', $this->configuration['tuts_exclude_groups_featured'] ? $this->configuration['tuts_exclude_groups_featured'] : '*', FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => true, 'unlimited' => '*', 'unlimitedLang' => 'all_user_groups' ), NULL, NULL, NULL, 'tuts_exclude_groups_featured' ) );
		$form->add( new \IPS\Helpers\Form\Node( 'tuts_cats_featured', isset( $this->configuration['tuts_cats_featured'] ) ? $this->configuration['tuts_cats_featured'] : 0, FALSE, array(
			'class'           => '\IPS\tutorials\Category',
			'zeroVal'         => 'all_categories',
			'permissionCheck' => 'view',
			'multiple'        => true
		) ) );

 		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		if ( isset( $this->configuration['tuts_exclude_groups_featured'] ) and is_array( $this->configuration['tuts_exclude_groups_featured'] ) )
		{
			if( !\IPS\Member::loggedIn()->inGroup( $this->configuration['tuts_exclude_groups_featured'] ) )
			{
				return '';
			}
		}

		if ( isset( $this->configuration['tuts_cats_featured'] ) and is_array( $this->configuration['tuts_cats_featured'] ) )
		{
			$where[] = array( 'a_cat IN (' . implode( ",", array_keys( $this->configuration['tuts_cats_featured'] ) ) . ')' );
		}

		$where[] = array( 'a_approved=1' );
		$where[] = array( 'a_featured=1' );

		$tutorials = \IPS\tutorials\Article::getItemsWithPermission( $where, NULL, isset( $this->configuration['tuts_to_show_featured'] ) ? $this->configuration['tuts_to_show_featured'] : 5 );

		if ( \count( $tutorials ) )
		{
			return $this->output( $tutorials );
		}
		else
		{
			return '';
		}
	}
}