<?php
/**
 * @brief		statsTutorials Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	tutorials
 * @since		13 Jan 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\tutorials\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * statsTutorials Widget
 */
class _statsTutorials extends \IPS\Widget\StaticCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'statsTutorials';
	
	/**
	 * @brief	App
	 */
	public $app = 'tutorials';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		if ( isset( $this->configuration['tuts_stats_exclude_groups'] ) and is_array( $this->configuration['tuts_stats_exclude_groups'] ) )
		{
			if( !\IPS\Member::loggedIn()->inGroup( $this->configuration['tuts_stats_exclude_groups'] ) )
			{
				return '';
			}
		}

		$stats				= array();
		$where[]			= array( 'a_approved = 1' );

		/* Tutorials count */
		$stats['total_tuts'] 		= \IPS\Db::i()->select( 'COUNT(*)', 'tutorials_articles', $where )->first();

		/* Comments count */
		$stats['total_comm'] 		= \IPS\Db::i()->select( 'COUNT(*)', 'tutorials_comments', array( 'comment_open=1' ) )->first();

		/* Comments count */
		$stats['total_rev'] 		= \IPS\Db::i()->select( 'COUNT(*)', 'tutorials_reviews', array( 'review_approved=1' ) )->first();

		$latestTutorial = NULL;
		$showComments 	= 0;
		$showReviews 	= 0;

		foreach ( \IPS\Db::i()->select( '*', 'tutorials_cats', NULL ) as $cat )
		{
			if( $cat['c_allow_comments'] )
			{
				$showComments = 1;
			}
		}

		foreach ( \IPS\Db::i()->select( '*', 'tutorials_cats', NULL ) as $cat )
		{
			if( $cat['c_allow_reviews'] )
			{
				$showReviews = 1;
			}
		}

		try
		{
			foreach ( \IPS\tutorials\Article::getItemsWithPermission( $where, NULL, 1 ) as $latestTutorial )
			{
				break;
			}
		}
		catch( \UnderflowException $ex )
		{
			$latestTutorial = NULL;
		}

		/* Display */		
		return $this->output( $stats, $latestTutorial, $showComments, $showReviews );
	}
}