<?php
/**
 * @brief		topContributorsTutorials Widget
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - SVN_YYYY Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Social Suite
 * @subpackage	tutorials
 * @since		24 Jun 2015
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\tutorials\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * topContributorsTutorials Widget
 */
class _topContributorsTutorials extends \IPS\Widget\StaticCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'topContributorsTutorials';
	
	/**
	 * @brief	App
	 */
	public $app = 'tutorials';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
		$form = parent::configuration( $form );

		$form->add( new \IPS\Helpers\Form\Number( 'top_tuts_submitter_tuts', isset( $this->configuration['top_tuts_submitter_tuts'] ) ? $this->configuration['top_tuts_submitter_tuts'] : 5, TRUE ) );
		$form->add( new \IPS\Helpers\Form\Select( 'top_tuts_submitter_exclude_groups', $this->configuration['top_tuts_submitter_exclude_groups'] ? $this->configuration['top_tuts_submitter_exclude_groups'] : '*', FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => true, 'unlimited' => '*', 'unlimitedLang' => 'all_user_groups' ), NULL, NULL, NULL, 'top_tuts_submitter_exclude_groups' ) );

		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		if ( isset( $this->configuration['top_tuts_submitter_exclude_groups'] ) and is_array( $this->configuration['top_tuts_submitter_exclude_groups'] ) )
		{
			if( !\IPS\Member::loggedIn()->inGroup( $this->configuration['top_tuts_submitter_exclude_groups'] ) )
			{
				return '';
			}
		}

		/* How many? */
		$limit = isset( $this->configuration['top_tuts_submitter_tuts'] ) ? $this->configuration['top_tuts_submitter_tuts'] : 5;
		
		/* Work out who has got the most reputation this week... */
		$topContributorsThisWeek = array();

		foreach ( \IPS\Db::i()->select( 'a_mid, a_id as rep', 'tutorials_articles', array( 'a_mid>0 AND a_date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1W' ) )->getTimestamp() ) ) as $rep )
		{
			if ( !isset( $topContributorsThisWeek[ $rep['a_mid'] ] ) )
			{
				$topContributorsThisWeek[ $rep['a_mid'] ] = \count( a_id );
			}
			else
			{
				$topContributorsThisWeek[ $rep['a_mid'] ] += \count( a_id );
			}
		}

		arsort( $topContributorsThisWeek );
		$topContributorsThisWeek = array_slice( $topContributorsThisWeek, 0, $limit, TRUE );
		
		/* Load their data */
		if( \count( $topContributorsThisWeek ) )
		{
			foreach ( \IPS\Db::i()->select( '*', 'core_members', \IPS\Db::i()->in( 'member_id', array_keys( $topContributorsThisWeek ) ) ) as $member )
			{
				\IPS\Member::constructFromData( $member );
			}
		}

		/* Display */
		return $this->output( $topContributorsThisWeek, $limit );
	}
}